"""

This is the main part of ERoaster

"""

# ----------------------------------------------------------------------
# Import functions I need 

from os import popen, environ, system, fork, lstat, chmod, unlink, pipe, system
from socket import socket, gethostbyname, SOCK_STREAM, AF_INET, gethostname, error
from string import strip, find, split, lower, rjust, atoi, atof, replace, digits, zfill
from sys import exit
from types import StringType

import sys

try:
    from gtk import GtkVBox, GtkNotebook, GtkFrame, GtkLabel, GtkHBox, GtkPixmap, GtkTable
except (RuntimeError, TypeError, NameError), detail:
    sys.stdout.write("\n[ERoaster startup] An error occured: %s\n\n" % detail)
    exit(1)
    
from gtk import GtkEntry, GtkCList, GtkScrolledWindow, GtkButton, GtkRadioButton
from gtk import GtkCheckButton, GtkSpinButton, GtkAdjustment, GtkProgressBar, GtkText
from gtk import GtkHScrollbar, GtkVScrollbar, GtkToolbar, GtkStatusbar, GtkFileSelection
from gtk import GtkOptionMenu, GtkMenu, GtkRadioMenuItem

from gtk import create_pixmap_from_xpm, rc_parse, mainloop, mainquit, timeout_add, timeout_remove
from gtk import events_pending, mainiteration

from gtk import SHADOW_ETCHED_OUT, FILL, EXPAND, DEST_DEFAULT_ALL, SELECTION_MULTIPLE
from gtk import JUSTIFY_RIGHT, TRUE, FALSE, POLICY_AUTOMATIC, SHRINK, ORIENTATION_HORIZONTAL
from gtk import TOOLBAR_BOTH, JUSTIFY_LEFT, POLICY_ALWAYS

from gnome.ui import GnomeApp, GnomeFileEntry, GnomeErrorDialog, GnomeAbout, GnomeQuestionDialog
from gnome.ui import STOCK_MENU_QUIT, STOCK_MENU_REDO, STOCK_MENU_UNDO, STOCK_MENU_ABOUT
from gnome.ui import UIINFO_ITEM_STOCK, UIINFO_SUBTREE
from gnome.ui import DOCK_ITEM_BEH_NORMAL, DOCK_TOP

from ConfigParser import ConfigParser
from select import select
from time import sleep, time, asctime, localtime

from tools import CheckDirectory, SigChldHandler, beautify, striplist, nicetime, du

from constants import title, version, frames, datacd, audiocd, imagecd, options, special, readcd, copycd
from constants import iconpath, deffile, True, False, gtkrcfile, ecat, modeFixateCD
from constants import DataCD, modeDataCDImage, AudioCD, ImageCD, modeCopyCD, modeReadCD, modeReadAudioCD
from constants import socketport, year, deviceRecorder, deviceReader, playerMpg123, playerXMMS
from constants import BlankCDAll, BlankCDMinimal, BlankCDLast, audioinfostring

import GtkExtra
import GDK
import os
import signal
import popen2
import re
import copy
import wave
import mp3info

# -----------------------------------------------------------------------------
# Main Object

class Application:

    def __init__(self, mkisofssupportsgui):

        # Some necessary Variables
        self.parser = ConfigParser()                        # The Configuration File Parser
        self.writermode = None                              # Data CD or Audio CD
        self.totalaudiosize = 0L                            # Total size of audio tracks
        self.totaltracklength = 0                           # Total length of audio tracks
        self.tracklength = {}                               # Dictionary containing the length of tracks
        self.tracksize = {}                                 # Dictionary containing the size of tracks
        self.datafilesize = {}                              # Dictionary containing the size of data files
        self.totaldatasize = 0L                             # Total size of data files
        self.selecteddatasize = 0L                          # Size of selected data files
        self.nraudiotracks = 0                              # Number of audio tracks
        self.burnwindow = None                              # There's no burn window at the beginning
        self.mkisofssupportsgui = mkisofssupportsgui        # Does mkisofs support -gui ?

        self.formataudio = re.compile("Track [ \d]+: [ \d]+ of [ \d]+ MB written.$")
        self.formataudiolong = re.compile("Track [ \d]+: [ \d]+ of [ \d]+ MB written \(fifo [ \d]+%\).$")
        self.formatdata = re.compile("Track [ \d]+: [ \d]+ MB written.$")
        self.formatdatalong = re.compile("Track [ \d]+: [ \d]+ MB written. [ \d\.]+% done, estimate finish")
        self.formatdatalongfifo = re.compile("Track [ \d]+: [ \d]+ MB written \(fifo [ \d]+%\). [ \d\.]+% done, estimate finish")
        self.formatdatafifo = re.compile("Track [ \d]+: [ \d]+ MB written \(fifo [ \d]+%\).")
        self.formatignore = re.compile("[\d\.]+% done, estimate|.*\d second")
        self.formatimage = re.compile("[\d\.]+% done, estimate finish")
        self.formatreadimage = re.compile("addr: [ \d]+ cnt: \d+")
        self.formatreadimageend = re.compile("end: [ \d]+")
        self.formatreadaudio = re.compile("[ \d]+%")
        self.formatignorecopy = re.compile("Track [ \d]+: [ \d]+ MB written.addr")
        self.formatreadaudioinfo = re.compile(audioinfostring)
        
        self.filemenu = [ UIINFO_ITEM_STOCK('Quit', None, self.Destroy, STOCK_MENU_QUIT),]
        self.editmenu = [ UIINFO_ITEM_STOCK('Select all', None, self.SelectAll, STOCK_MENU_REDO),
                          UIINFO_ITEM_STOCK('Unselect all', None, self.UnSelectAll, STOCK_MENU_UNDO) ]
        self.helpmenu = [ UIINFO_ITEM_STOCK('About ...', None, self.About, STOCK_MENU_ABOUT),]
        self.menus = [ UIINFO_SUBTREE('File', self.filemenu),
                       UIINFO_SUBTREE('Edit', self.editmenu),
                       UIINFO_SUBTREE('Help', self.helpmenu)]

	# Initialize the window
	self.window = GnomeApp("ERoaster", "%s %s" % (title, version))
	self.window.connect("destroy", self.Destroy)
	self.window.set_border_width(1)

	# Initialize the Main Box
	self.metabox = GtkVBox()
	self.metabox.show()
        self.window.set_contents(self.metabox)
        
        # Create the menu bar
        self.window.create_menus(self.menus)

	# Create the toolbar
        self.toolbar =  GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_BOTH)
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/image.xpm" % iconpath)
        self.toolbar.append_item("Create Image", "Create Image", "", GtkPixmap(pix, mask), self.CreateImage)
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/burn.xpm" % iconpath)
        self.toolbar.append_item("Burn CD", "Burn CD", "", GtkPixmap(pix, mask), self.BurnCD)
        self.toolbar.append_space()
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/select.xpm" % iconpath)
        self.toolbar.append_item("Select All", "Select All", "", GtkPixmap(pix, mask), self.SelectAll)
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/unselect.xpm" % iconpath)
        self.toolbar.append_item("Unselect All", "Unselect All", "", GtkPixmap(pix, mask), self.UnSelectAll)
        self.toolbar.append_space()
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/quit.xpm" % iconpath)
        self.toolbar.append_item("Quit", "Quit ECLiPt Roaster", "", GtkPixmap(pix, mask), self.Destroy)
        self.toolbar.show()
        self.window.add_toolbar(self.toolbar, "My Toolbar", DOCK_ITEM_BEH_NORMAL, DOCK_TOP, 1, 1, 0)

        # Create a status bar
        self.statusbar = GtkStatusbar()
        self.statusbar.show()
        self.window.set_statusbar(self.statusbar)
        self.statusbar.push(self.statusbar.get_context_id("main"), " Welcome to the %s %s (c) %s by Martin Preishuber" % (title, version, year))
        
	# Box which holds the tabs
	self.mainbox = GtkVBox(spacing = 5)
	self.mainbox.set_border_width(5)
	self.metabox.pack_start(self.mainbox)
	self.mainbox.show()

	self.notebook = GtkNotebook()
	self.mainbox.pack_start(self.notebook)
	self.notebook.show()

        self.setstatustip = False
        self.notebook.connect("switch_page", self.ShowStatusbarTips)
        
	# Create the Frames:
	for i in range(len(frames)):
	    self.CreateFrame(frames[i])

	# Define entry fields & stuff which gets set per default
	self.LoadOptions()

	# Show main window
	self.window.show()

        signal.signal(signal.SIGCHLD, SigChldHandler)
        self.setstatustip = True

    # Set tips in the status bar
    def ShowStatusbarTips(self, notebook, object, index):
        if self.setstatustip == True:
            if (index == 0):
                message = "Drag & drop some files to the \"Files / Directories\"-list and click AutoSelect"
            elif (index == 1):
                message = "Drag & drop wave- or mp3-files to the list and order it via Drag & Drop"
            elif (index == 2):
                message = "Select an image and click on \"Image info\" for checking"
            elif (index == 3):
                message = "Read the a CD image"
            elif (index == 4):
                message = "Copy a CD"
            elif (index == 5):
                message = "Perform special operations on CDs"
            elif (index == 6):
                message = "Set the general preferences"
                
            self.statusbar.push(self.statusbar.get_context_id("main"), " %s" % message)
        
    # Create a box consisting of a Xpm and a Label
    def XpmLabelBox(self, text, xpm = None):
	label = GtkLabel(text)
	label.set_padding(2, 2)
	label.show()

	labelbox = GtkHBox()

        if (xpm != None):
            pix, mask = create_pixmap_from_xpm(self.window, None, xpm)
            pixmap = GtkPixmap(pix, mask)
            pixmap.show()
            labelbox.add(pixmap)

	labelbox.add(label)
	labelbox.show()

	return labelbox

    # Define a About Box
    def About(self, args):
        GnomeAbout(title, version, 'May be distributed under the terms of the GPL2', ['Martin Preishuber'], 'This is a frontend for writing and copying CDs using cdrecord & mkisofs.').show()

    # Create the various frames
    def CreateFrame(self, name):

	frame = GtkFrame(name)
	frame.set_border_width(5)
	frame.set_shadow_type(SHADOW_ETCHED_OUT)
	frame.show()
	
	if (name == datacd):
	    box = self.XpmLabelBox(name, "%s/master.xpm" % iconpath)
	    self.BurnData(frame)
	elif (name == audiocd):
	    box = self.XpmLabelBox(name, "%s/audio.xpm" % iconpath)
	    self.BurnAudio(frame)
	elif (name == imagecd):
	    box = self.XpmLabelBox(name, "%s/iso.xpm" % iconpath)
	    self.BurnImage(frame)
        elif (name == readcd):
	    box = self.XpmLabelBox(name, "%s/read.xpm" % iconpath)
	    self.ReadCDFrame(frame)
        elif (name == copycd):
	    box = self.XpmLabelBox(name, "%s/copy.xpm" % iconpath)
	    self.CopyCDFrame(frame)
        elif (name == special):
	    box = self.XpmLabelBox(name, "%s/iso.xpm" % iconpath)
	    self.SpecialOperation(frame)
	elif (name == options):
	    box = self.XpmLabelBox(name, "%s/options.xpm" % iconpath)
	    self.Options(frame)
            
        self.notebook.append_page(frame, box)

    # Burn CD (menu entry)
    def BurnCD(self, args):
        if (self.burnwindow != None):
            GnomeErrorDialog("Close the output window first !")
            return
        page = self.notebook.get_current_page()

        if (frames[page] == options) or (frames[page] == readcd):
            GnomeErrorDialog("You can't burn a CD at this point !")
            return

        warningdialog = False
        
        if (frames[page] == datacd):
            self.writermode = DataCD
            if (len(self.datafiles.selection)) == 0:
                GnomeErrorDialog("You have to select files & directories first.")
                return

            selectedsize = atoi(self.datasizesel.get_text()[:-3])
            if self.cdsize650.active:
                cdsize = 650
            elif self.cdsize700.active:
                cdsize = 700

            if selectedsize > cdsize:
                warningdialog = True
                GnomeQuestionDialog("Warning: You selected %s MB which is larger then your selected CD size (%s MB).\nAre you sure you want to continue ?" % (selectedsize, cdsize), self.CallBackTooBig)

        elif (frames[page] == audiocd):
            self.writermode = AudioCD
        elif (frames[page] == imagecd):
            self.writermode = ImageCD
        elif (frames[page] == copycd):
            self.writermode = modeCopyCD

        if warningdialog == False:
            self.SelectWritingOptions()

    def CallBackTooBig(self, result):
        if result == 0:
            self.SelectWritingOptions()
        
    # --------------------------------------------------------------------------
    # Open a window asking some questions about the CD creation
    def SelectWritingOptions(self):

	self.woptwindow = GnomeApp("ERoasterOptions", "CD-R(W) Options")
	self.woptwindow.set_border_width(1)
        self.woptwindow.show()

        vbox = GtkVBox(spacing = 5)
        vbox.set_border_width(0)
        vbox.show()

        woptframe = GtkFrame(" Settings ")
        woptframe.set_border_width(5)
        woptframe.set_shadow_type(SHADOW_ETCHED_OUT)
        woptframe.show()

        self.formatCDR = GtkRadioButton(None, "CD-R")
	self.formatCDR.show()
	self.formatCDRW = GtkRadioButton(self.formatCDR, "CD-RW")
	self.formatCDRW.show()
        if self.tw_mode == 0:
            self.formatCDR.set_active(1)
        elif self.tw_mode == 1:
            self.formatCDRW.set_active(1)
            
	self.fixate = GtkCheckButton("fixate CD when done")
        self.fixate.set_active(self.tw_fixate)
        if self.writermode == modeCopyCD:
            self.fixate.set_sensitive(FALSE)
        else:
            self.fixate.set_sensitive(TRUE)            
	self.fixate.show()

        self.multisession = GtkCheckButton("multisession CD")
        # self.multisession.set_active(self.tw_multisession) # FIXME
        self.multisession.set_active(0)
        if self.writermode == modeCopyCD:
            self.multisession.set_sensitive(FALSE)
        else:
            self.multisession.set_sensitive(TRUE)
        self.multisession.set_sensitive(FALSE) # FIXME: remove
        
        self.multisession.show()
        
        wopttable = GtkTable(3, 2)
        wopttable.attach(self.formatCDR, 0, 1, 0, 1, 0, 0, 5)
        wopttable.attach(self.formatCDRW, 1, 2, 0, 1, 0, 0, 5)
        wopttable.attach(self.fixate, 0, 2, 1, 2, FILL, 0, 5)
        wopttable.attach(self.multisession, 0, 2, 2, 3, FILL, 0, 5)
        wopttable.show()

        woptframe.add(wopttable)

        vbox.pack_start(woptframe, expand = FALSE)

        goBtn = GtkButton()
        box = self.XpmLabelBox(" Go ", "%s/ok.xpm" % iconpath)
	goBtn.add(box)
	goBtn.connect('clicked', self.BurnWriterOptions)
        goBtn.show()
        cancelBtn = GtkButton()
        box = self.XpmLabelBox(" Cancel ", "%s/cancel.xpm" % iconpath)
	cancelBtn.add(box)
	cancelBtn.connect('clicked', self.CloseWriterOptions)
        cancelBtn.show()

        labLeft = GtkLabel()
        labLeft.show()
        labRight = GtkLabel()
        labRight.show()

        buttontab = GtkTable(1, 4)
        buttontab.attach(labLeft, 0, 1, 0, 1, EXPAND, 0, 5, 5)
        buttontab.attach(goBtn, 1, 2, 0, 1, 0, 0, 5, 5)
        buttontab.attach(cancelBtn, 2, 3, 0, 1, 0, 0, 5, 5)
        buttontab.attach(labRight, 3, 4, 0, 1, EXPAND, 0, 5, 5)
        buttontab.show()
        
        vbox.pack_start(buttontab)
        
        self.woptwindow.set_contents(vbox)

    # Close the writer options window and start burnin'
    def BurnWriterOptions(self, args):
        if self.formatCDR.active:
            self.tw_mode = 0
        elif self.formatCDRW.active:
            self.tw_mode = 1
        self.tw_fixate = self.fixate.active
        self.tw_multisession = self.multisession.active
	self.woptwindow.destroy()
        self.woptwindow = None

        if (frames[self.notebook.get_current_page()] == audiocd):
            self.ConvertMp3ToWav()
            self.Normalize()
            
        self.BurnCDNow()
        
    # Close the writer options window
    def CloseWriterOptions(self, args):
	self.woptwindow.destroy()
        self.woptwindow = None

    # --------------------------------------------------------------------------
    # Create Image (menu entry)    
    def CreateImage(self, args):
        if (self.imageentry.gtk_entry().get_text() == ""):
            GnomeErrorDialog("Please select a image path in the preferences tab first !")
            return
        if (not os.path.isdir(self.imageentry.gtk_entry().get_text())):
            GnomeErrorDialog("Error: \"%s\" is not a valid directory !" % self.imageentry.gtk_entry().get_text())
            return
        if (self.burnwindow != None):
            GnomeErrorDialog("Close the output window first !")
            return
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.writermode = modeDataCDImage
        elif (frames[page] == imagecd) or (frames[page] == audiocd) or (frames[page] == options) or (frames[page] == readcd) or (frames[page] == special) or (frames[page] == copycd):
            GnomeErrorDialog("You can't create any image at this point !")
            return

        if (len(self.datafiles.selection)) == 0:
            GnomeErrorDialog("You have to select files & directories first.")
            return

        self.filenamewindow = GnomeApp("ERoasterFileName", "Enter Filename")
	self.filenamewindow.set_border_width(1)
        self.filenamewindow.show()

        vbox = GtkVBox(spacing = 5)
        vbox.set_border_width(0)
        vbox.show()

        fileframe = GtkFrame(" Filename ")
        fileframe.set_border_width(5)
        fileframe.set_shadow_type(SHADOW_ETCHED_OUT)
        fileframe.show()

        fileLabel = GtkLabel("Filename: ")
        fileLabel.show()

	self.imagefilename = GnomeFileEntry("imagename")
	self.imagefilename.set_usize(350, -1)
        self.imagefilename.show()

        if self.volumeid.get_text() != "":
            isofilename = self.volumeid.get_text()
        else:
            isofilename = "ECLiPt Roaster - %s" % asctime(localtime(time()))

        imagepath = self.imageentry.gtk_entry().get_text()
        if imagepath[-1] == "/":
            isofilename = "%s%s.iso" % (imagepath, isofilename)
        else:
            isofilename = "%s/%s.iso" % (imagepath, isofilename)

        self.imagefilename.gtk_entry().set_text(isofilename)

        filetab = GtkTable(1, 2)
        filetab.attach(fileLabel, 0, 1, 0, 1, 0, 0, 5, 5)
        filetab.attach(self.imagefilename, 1, 2, 0, 1, 0, 0, 5, 5)
        filetab.show()

        okBtn = GtkButton()
        box = self.XpmLabelBox(" OK ", "%s/ok.xpm" % iconpath)
	okBtn.add(box)
        okBtn.connect('clicked', self.CreateImageOK)
        okBtn.show()
        cancelBtn = GtkButton()
        box = self.XpmLabelBox(" Cancel ", "%s/cancel.xpm" % iconpath)
	cancelBtn.add(box)
	cancelBtn.connect('clicked', self.CreateImageCancel)
        cancelBtn.show()

        labLeft = GtkLabel()
        labLeft.show()
        labRight = GtkLabel()
        labRight.show()

        buttontab = GtkTable(1, 4)
        buttontab.attach(labLeft, 0, 1, 0, 1, EXPAND, 0, 5, 5)
        buttontab.attach(okBtn, 1, 2, 0, 1, 0, 0, 5, 5)
        buttontab.attach(cancelBtn, 2, 3, 0, 1, 0, 0, 5, 5)
        buttontab.attach(labRight, 3, 4, 0, 1, EXPAND, 0, 5, 5)
        buttontab.show()

        fileframe.add(filetab)

        vbox.pack_start(fileframe, expand = FALSE)
        vbox.pack_start(buttontab)

        self.filenamewindow.set_contents(vbox)
                        
    # Create the CD Image (OK button pressed)
    def CreateImageOK(self, args):
        self.fileNameCDImage = self.imagefilename.gtk_entry().get_text()
	self.filenamewindow.destroy()
        self.filenamewindow = None
        self.BurnCDNow()

    # Create the CD Image (Cancel button pressed)
    def CreateImageCancel(self, args):
	self.filenamewindow.destroy()
        self.filenamewindow = None
        
    # Burn Image
    def BurnImageNow(self, args):

        if (self.burnwindow != None):
            GnomeErrorDialog("Close the output window first !")
            return

        self.writermode = ImageCD
        self.BurnCDNow()

    # Burn CD depending of which type
    def BurnCDNow(self):

        if self.tw_mode == 0:
            speed = self.speed.get_value_as_int()
        else:
            speed = self.speedRW.get_value_as_int()
        
        if (self.channel.get_text() == "") or (self.id.get_text() == "") or (self.lun.get_text() == ""):
            GnomeErrorDialog("Invalid recorder settings !")
            return

        if (self.writermode == DataCD) or (self.writermode == modeDataCDImage):

            if (self.mkisofssupportsgui == True):
                command = "(mkisofs -gui -v"
            else:
                command = "(mkisofs -v"
            if self.diralloc.active: command = "%s -D" % command
            if self.fullchar.active: command = "%s -l" % command
            if self.allowdots.active: command = "%s -L" % command
            if self.followlinks.active: command = "%s -f" % command
            if self.joliet.active: command = "%s -J" % command
            if self.ratrock.active: command = "%s -r" % command
            if self.rockridge.active: command = "%s -R" % command
            if self.periods.active: command = "%s -d" % command
            volumeid = self.volumeid.get_text()
            if (volumeid != ""):
                command = "%s -V \"%s\"" % (command, volumeid)

            if (self.writermode == modeDataCDImage):
                command = "%s -o \"%s\"" % (command, self.fileNameCDImage)

            command = "%s -graft-points" % command
            
            for i in range(len(self.datafiles.selection)):
                filename = self.datafiles.get_text(self.datafiles.selection[i], 0)
                if os.path.isdir(filename):
                    if (filename[-1] == "/"):
                        filename = filename[:-1]
                    filenameshort = split(filename, "/")[-1]
                    command = "%s \"/%s/\"=\"%s\"" % (command, filenameshort, filename)
                else:
                    command = "%s \"%s\"" % (command, filename)

            if (self.writermode == DataCD):
                command = "%s | cdrecord -v" % command
                if self.tw_fixate == False: command = "%s -nofix" % command
                if self.tw_multisession == True: command = "%s -multi" % command
                if self.dummy.active: command = "%s -dummy" % command
                if self.eject.active: command = "%s -eject" % command
                command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
                command = "%s dev=%d,%d,%d speed=%d -pad -data -) 2>&1 | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, ecat)
            elif (self.writermode == modeDataCDImage):
                command = "%s) 2>&1 | %s" % (command, ecat)

        elif (self.writermode == AudioCD):
            
            if (self.songlist.rows) == 0:
                GnomeErrorDialog("You have to select some audio-files first.")
                return

            command = "cdrecord -audio -v"
            if self.tw_fixate == False: command = "%s -nofix" % command
            if self.tw_multisession == True: command = "%s -multi" % command
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%d" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed)
            if self.swab.active: command = "%s -swab" % command
            if self.pad.active: command = "%s -pad" % command

            line = 0
            try:
                filename = self.songlist.get_text(line, 1)
            except:
                filename = None
            while (filename != None):
                command = "%s \"%s\"" % (command, filename)
                self.nraudiotracks = line + 1                                   # Numbers don't start at 0 !
                line = line + 1
                try:
                    filename = self.songlist.get_text(line, 1)
                except:
                    filename = None
            command = "%s 2>&1 | %s" % (command, ecat)

        elif (self.writermode == ImageCD):
            
            image = self.cdimage.gtk_entry().get_text()
            if (image == "") or (os.path.isdir(image)):
                GnomeErrorDialog("The image file \"%s\" is not valid." % image)
                return

            command = "cdrecord -v"
            if self.tw_fixate == False: command = "%s -nofix" % command
            if self.tw_multisession == True: command = "%s -multi" % command
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%s %s 2>&1 | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, image, ecat)

        elif (self.writermode == modeCopyCD):
            command = "(readcd -dev=%d,%d,%d -f /dev/stdout " % ( int(self.channelReader.get_text()), int(self.idReader.get_text()), int(self.lunReader.get_text()))
            command = "%s | cdrecord -v" % command
            if self.tw_fixate == False: command = "%s -nofix" % command
            if self.tw_multisession == True: command = "%s -multi" % command
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%d -pad -data -) 2>&1 | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, ecat)

        elif (self.writermode == BlankCDAll) or (self.writermode == BlankCDMinimal) or (self.writermode == BlankCDLast):

            speed = self.speedRW.get_value_as_int()

            if (self.writermode == BlankCDAll):
                blankmode = "disc"
            elif (self.writermode == BlankCDMinimal):
                blankmode = "fast"
            elif (self.writermode == BlankCDLast):
                blankmode = "session"
                
            command = "cdrecord -v"
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%s -blank %s 2>&1 | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, blankmode, ecat)

        elif (self.writermode == modeFixateCD):

            speed = self.speedRW.get_value_as_int()

            command = "cdrecord -v"
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%s -fix 2>&1 | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, ecat)

        self.BurnWindow(command)

    # Open a new window which contains the cdrecord & mkisofs output
    def BurnWindow(self, command):

	self.burnwindow = GnomeApp("ERoasterOutput", "%s Output" % title)
	self.burnwindow.set_border_width(1)

	self.burnbox = GtkVBox(spacing = 5)
	self.burnbox.set_border_width(5)
	self.burnbox.show()

        # Create a frame containing tracknumber & progress bar

        frame = GtkFrame(label = " Progress Bar ")
        frame.set_border_width(3)
        frame.set_shadow_type(SHADOW_ETCHED_OUT)
        frame.show()

        table = GtkTable(1, 7)

        label = GtkLabel("Track#:")
        label.show()

        self.track = GtkEntry()
        self.track.set_usize(55, -1)
        self.track.set_editable(FALSE)
        self.track.show()

        self.progressbar = GtkProgressBar()
        self.progressbar.set_usize(180, -1)
        self.progressbar.set_format_string("")
        self.progressbar.set_show_text(TRUE)
        self.progressbar.show()

        fifolabel = GtkLabel("FIFO:")
        fifolabel.show()

        self.fifo = GtkEntry()
        self.fifo.set_usize(40, -1)
        self.fifo.set_editable(FALSE)
        self.fifo.show()

        donelabel = GtkLabel("Done:")
        donelabel.show()
        
        self.estimate = GtkEntry()
        self.estimate.set_usize(160, -1)
        self.estimate.set_editable(FALSE)
        self.estimate.set_text("(not estimated)")
        self.estimate.show()
        
        table.attach(label, 0, 1, 0, 1, 0, 0, 5, 5)
        table.attach(self.track, 1, 2, 0, 1, 0, 0, 5, 5)
        table.attach(self.progressbar, 2, 3, 0, 1, 0, 0, 5, 5)
        table.attach(fifolabel, 3, 4, 0, 1, 0, 0, 5, 5)
        table.attach(self.fifo, 4, 5, 0, 1, 0, 0, 5, 5)
        table.attach(donelabel, 5, 6, 0, 1, 0, 0, 5, 5)
        table.attach(self.estimate, 6, 7, 0, 1, 0, 0, 5, 5)
        
        table.show()
        
        frame.add(table)
        
        self.burnbox.pack_start(frame, expand = FALSE)
        
	# Creata frame containing the rest of the output

	outputframe = GtkFrame(label = " CDRecord & MkIsoFS Output ")
	outputframe.set_border_width(3)
        outputframe.set_shadow_type(SHADOW_ETCHED_OUT)
	outputframe.show()
	
        table = GtkTable(2, 2)
        table.set_border_width(10)
        table.set_row_spacing(0, 2)
        table.set_col_spacing(0, 2)
        table.show()

	outputframe.add(table)
        
        self.cdtext = GtkText()
        self.cdtext.set_usize(585, 250)
        self.cdtext.set_editable(FALSE)
        self.cdtext.set_word_wrap(FALSE)
        table.attach(self.cdtext, 0, 1, 0, 1, 0, 0)
        self.cdtext.show()

        hscrollbar = GtkHScrollbar(self.cdtext.get_hadjustment())
        table.attach(hscrollbar, 0, 1, 1, 2, FILL, 0)
        hscrollbar.show()

        vscrollbar = GtkVScrollbar(self.cdtext.get_vadjustment())
        table.attach(vscrollbar, 1, 2, 0, 1, 0, FILL)
        vscrollbar.show()

	self.burnbox.pack_start(outputframe)

	# Insert a table with one button to close the window

        closeframe = GtkFrame()
        closeframe.set_border_width(3)
        closeframe.set_shadow_type(SHADOW_ETCHED_OUT)
        closeframe.show()

	buttontab = GtkTable(1, 3)

	dummylabel = GtkLabel()
	dummylabel.show()
	buttontab.attach(dummylabel, 0, 1, 0, 1, FILL|EXPAND, 0, 5)

	self.closebutton = GtkButton()
	self.closebox = self.XpmLabelBox(" Close Window ", "%s/close.xpm" % iconpath)
	self.closebutton.add(self.closebox)
        self.closebutton.set_sensitive(FALSE)
	self.closebutton.show()
    	buttontab.attach(self.closebutton, 1, 2, 0, 1, 0, 0, 0, 5)

	dummylabel = GtkLabel()
	dummylabel.show()
	buttontab.attach(dummylabel, 2, 3, 0, 1, FILL|EXPAND, 0, 5)

	buttontab.show()

        closeframe.add(buttontab)
        
	self.burnbox.pack_start(closeframe, expand = FALSE)

	self.burnwindow.set_contents(self.burnbox)
        self.burnwindow.show()

        pid = os.fork()
        if (pid == 0):
            connected = FALSE
            while (connected == FALSE):
                try:
                    self.client = socket(AF_INET, SOCK_STREAM)
                    self.client.connect(gethostbyname(gethostname()), socketport)
                    connected = TRUE
                except error, detail:
                    sleep(0.1)
            sys.stderr = sys.stdout
            if self.writermode == modeReadAudioCD:
                fileList = []
                for i in range(len(self.audiotracklist)):
                    if self.audiotracklist[i].active:
                        fileList.append(i + 1)
            else:
                fileList = ["dummy"]

            for i in range(len(fileList)):
                if self.writermode == modeReadAudioCD:
                    fullcommand = command % (fileList[i], fileList[i], "/opt/images/", fileList[i])
                    pipe = popen2.popen2(fullcommand)
                    fileString = "%s %d of %d\n" % (audioinfostring, i + 1, len(fileList))
                    self.client.send(rjust(str(len(fileString)), 4))
                    self.client.send(fileString)
                else:
                    pipe = popen2.popen2(command)
                line = None
                while (line != ""):
                    line = pipe[0].readline()
                    if (line != ""):
                        self.client.send(rjust(str(len(line)), 4))
                        self.client.send(line)
                pipe[0].close()
                pipe[1].close()
                pipe = None
            self.client.close()
            os._exit(0)
        else:
            self.server = socket(AF_INET, SOCK_STREAM)
            self.server.bind(gethostbyname(gethostname()), socketport)
            self.server.listen(1)
            self.connection, self.address = self.server.accept()

        self.delhandler = timeout_add(250, self.TimeOutHandler)

    # Handler to check the pipe output
    def TimeOutHandler(self):
        while 1:
            [iwtd, owtd, ewtd] = select([self.connection], [], [], 0)
            if (len(iwtd) == 0):
                return TRUE
            else:
                packet = self.connection.recv(4)
                if (packet == ""):
                    self.closebutton.set_sensitive(TRUE)
                    self.closebutton.connect("pressed", self.BurnWindowDestroy)
                    timeout_remove(self.delhandler)
                    return TRUE
                length = atoi(packet)
                packet = self.connection.recv(length)
                if not packet: return FALSE
                splitted = split(packet, "\n")
                for i in range(len(splitted)):
                    line = strip(splitted[i])
                    if (line != ""):

                        # Format lines for writing audio CDs
                        
                        if (self.writermode == AudioCD):

                            if self.formataudio.match(line):
                                self.track.set_text("%s of %s" % (str(atoi(line[6:8])), str(self.nraudiotracks)))
                                formatstr = "%s of %sMB (Total %d MB)" % (strip(line[10:13]), strip(line[17:20]), round(float(self.totalaudiosize) / (1024 * 1024)))
                                if (atof(line[10:13]) <= atof(line[17:20])):
                                    self.progressbar.set_format_string(formatstr)
                                    if atof(line[17:20]) > 0:
                                        self.progressbar.update(atof(line[10:13]) / atof(line[17:20]))
                            elif self.formataudiolong.match(line):
                                self.track.set_text("%s of %s" % (str(atoi(line[6:8])), str(self.nraudiotracks)))
                                formatstr = "%s of %sMB (Total %d MB)" % (strip(line[10:13]), strip(line[17:20]), round(float(self.totalaudiosize) / (1024 * 1024)))
                                if (atof(line[10:13]) <= atof(line[17:20])):
                                    self.progressbar.set_format_string(formatstr)
                                    if atof(line[17:20]) > 0:
                                        self.progressbar.update(atof(line[10:13]) / atof(line[17:20]))
                                self.fifo.set_text("%s%%" % strip(line[38:41]))
                            elif (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

                        # Format lines for reading CDs
                        
                        elif (self.writermode == modeReadCD):
                            
                            if self.formatreadimage.match(line):
                                cursize = atoi(strip(line[6:14])) * 2
                                formatstr = "%p%% (" + str(cursize / 1024) + " of " + str(self.tmpimagesize / 1024) + "MB)"
                                self.progressbar.set_format_string(formatstr)
                                if (float(self.tmpimagesize) / 1024) > 0:
                                    self.progressbar.update((float(cursize) / 1024) / (float(self.tmpimagesize) / 1024))
                            elif self.formatreadimageend.match(line):
                                self.tmpimagesize = atoi(strip(line[5:])) * 2
                                formatstr = "%p%% (0 of " + str(self.tmpimagesize / 1024) + "MB)"
                                self.progressbar.set_format_string(formatstr)
                            elif (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

                        # Format lines for reading audio CDs
                        
                        elif (self.writermode == modeReadAudioCD):

                            if self.formatreadaudio.match(line):
                                percent = atof(line[:find(line, "%")])
                                formatstr = "%p%% (File " + self.filenumber + ")"
                                self.progressbar.set_format_string(formatstr)
                                self.progressbar.update(percent / 100)
                            elif self.formatreadaudioinfo.match(line):
                                self.filenumber = line[len(audioinfostring) + 1:]
                            elif (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

                        # Format lines for writing data CDs
                        
                        elif (self.writermode == DataCD):

                            if self.formatdata.match(line):
                                self.track.set_text(str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                                if (atof(strip(line[9:13])) <= round(float(self.selecteddatasize) / 1024)):
                                    self.progressbar.set_format_string(formatstr)
                                    if round(float(self.selecteddatasize) / 1024) > 0:
                                        self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                            elif self.formatdatalongfifo.match(line):
                                self.track.set_text(str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                                if (atof(strip(line[9:13])) <= round(float(self.selecteddatasize) / 1024)):
                                    self.progressbar.set_format_string(formatstr)
                                    if round(float(self.selecteddatasize) / 1024) > 0:
                                        self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                                self.fifo.set_text("%s%%" % strip(line[31:34]))
                                self.estimate.set_text(line[67:])
                            elif self.formatdatafifo.match(line):
                                self.track.set_text(str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                                if (atof(strip(line[9:13])) <= round(float(self.selecteddatasize) / 1024)):
                                    self.progressbar.set_format_string(formatstr)
                                    if round(float(self.selecteddatasize) / 1024) > 0:
                                        self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                                self.fifo.set_text("%s%%" % strip(line[31:34]))
                            elif self.formatdatalong.match(line):
                                self.track.set_text(str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                                if (atof(strip(line[9:13])) <= round(float(self.selecteddatasize) / 1024)):
                                    self.progressbar.set_format_string(formatstr)
                                    if round(float(self.selecteddatasize) / 1024) > 0:
                                        self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                                self.estimate.set_text(line[55:])
                            elif (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

                        # Format lines for formatting CDs
                        
                        elif (self.writermode == BlankCDAll) or (self.writermode == BlankCDMinimal) or (self.writermode == BlankCDLast) or (self.writermode == modeFixateCD):
                            if (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

                        # Format lines for copying CDs

                        elif (self.writermode == modeCopyCD):
                            if self.formatreadimageend.match(line):
                                self.tmpimagesize = atoi(strip(line[5:])) * 2
                                formatstr = "%p%% (0 of " + str(self.tmpimagesize / 1024) + "MB)"
                                self.progressbar.set_format_string(formatstr)
                            elif (self.formatdatafifo.match(line)):
                                self.track.set_text(str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.tmpimagesize) / 1024))) + " MB)"
                                self.progressbar.set_format_string(formatstr)
                                if round(float(self.tmpimagesize) / 1024) > 0:
                                    self.progressbar.update(atof(strip(line[9:13])) / round(float(self.tmpimagesize) / 1024))
                                self.fifo.set_text("%s%%" % strip(line[31:34]))
                            elif (not self.formatignorecopy.match(line)) and (not self.formatreadimage.match(line)) and (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)
                                
                        # Format lines for creating CD images

                        elif (self.writermode == ImageCD):
                            if self.formataudio.match(line):
                                self.track.set_text("%s" % str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[10:13]) + " of " + strip(line[17:20]) + " MB)"
                                if (atof(line[10:13]) <= atoi(line[17:20])):
                                    self.progressbar.set_format_string(formatstr)
                                    if atof(line[17:20]) > 0:
                                        self.progressbar.update(atof(line[10:13]) / atof(line[17:20]))
                                
                            elif self.formataudiolong.match(line):
                                self.track.set_text("%s" % str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[10:13]) + " of " + strip(line[17:20]) + " MB)"
                                if (atof(line[10:13]) <= atof(line[17:20])):
                                    self.progressbar.set_format_string(formatstr)
                                    if atof(line[17:20]) > 0:
                                        self.progressbar.update(atof(line[10:13]) / atof(line[17:20]))
                                self.fifo.set_text("%s%%" % strip(line[38:41]))
                            elif (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

                        # Format lines for creating Image files
                        
                        elif (self.writermode == modeDataCDImage):

                            if self.formatimage.match(line):
                                formatstr = "%p%%"
                                self.progressbar.set_format_string(formatstr)
                                self.progressbar.update(atof(strip(line[:find(line, ".")])) / 100)
                                self.estimate.set_text(line[find(line, ".") + 27:])
                            elif (not self.formatignore.match(line)):
                                self.cdtext.insert_defaults("%s\n" % line)

    # Close the Burn window
    def BurnWindowDestroy(self, args):
        self.connection.close()
        self.server.close()
	self.burnwindow.destroy()
        self.burnwindow = None

    # Normalize wav-files
    def Normalize(self):
        
        if self.normalize.active:

            win = GnomeApp("ERoasterNormalizeWait", "Please Wait")
            win.set_border_width(5)
            win.show()

            label = GtkLabel(" Normalizing wav-files - This may take a minute or two... ")
            label.show()

            waitframe = GtkFrame(" Please Wait ")
            waitframe.set_border_width(5)
            waitframe.set_shadow_type(SHADOW_ETCHED_OUT)
            waitframe.add(label)
            waitframe.show()

            vbox = GtkVBox(spacing = 5)
            vbox.set_border_width(0)
            vbox.pack_start(waitframe)
            vbox.show()

            win.set_contents(vbox)

            # FIXME: make sure window is being drawn
            while events_pending():
                mainiteration()

            command = "normalize -q"
            line = 0
            try:
                filename = self.songlist.get_text(line, 1)
            except detail:
                filename = None
            while (filename != None):

                command = "%s \"%s\"" % (command, filename)

                line = line + 1
                try:
                    filename = self.songlist.get_text(line, 1)
                except:
                    filename = None

            system(command)

            win.destroy()
            win = None

    # Set various options for CDRecord & MkIsoFS
    def Options(self, frame):

        # A table containing various options
        opttable = GtkTable(3, 1)
        opttable.show()

        # Filesystem options
        fsframe = GtkFrame(" Filesystem (Data & Audio) ")
        fsframe.set_border_width(5)
        fsframe.set_shadow_type(SHADOW_ETCHED_OUT)
        fsframe.show()

        # Create a table containing the check buttons
	fstab = GtkTable(5, 2)
	fstab.show()

	self.diralloc = GtkCheckButton("Disable deep directory relocation")
	self.diralloc.show()
	self.fullchar = GtkCheckButton("Allow full 32 character filenames")
	self.fullchar.show()
	self.allowdots = GtkCheckButton("Allow file to start with '.'")
	self.allowdots.show()
	self.followlinks = GtkCheckButton("Follow symbolic links")
	self.followlinks.show()
	self.joliet = GtkCheckButton("Generate Joliet directory information")
	self.joliet.show()
	self.ratrock = GtkCheckButton("Generate rationalized Rock Ridge directory information")
	self.ratrock.show()
	self.rockridge = GtkCheckButton("Generate Rock Ridge directory information")
	self.rockridge.show()
	self.periods = GtkCheckButton("Omit trailing periods from filenames")
	self.periods.show()
	self.pad = GtkCheckButton("Pad audio tracks to a multiple of 2352 bytes")
	self.pad.show()
	self.swab = GtkCheckButton("Audio data source is byte-swapped (little-endian/Intel)")
	self.swab.show()

        fifolabel = GtkLabel("FIFO Size:")
        fifolabel.show()
        self.fifosize = GtkSpinButton(GtkAdjustment(4,2,64,1,10,0), 0, 0)
        self.fifosize.show()
        mblabel = GtkLabel("MB ")
        mblabel.show()
        empty = GtkLabel()
        empty.show()

        fifobox = GtkHBox(spacing = 5)
        fifobox.pack_start(fifolabel, expand = FALSE)
        fifobox.pack_start(self.fifosize, expand = FALSE)
        fifobox.pack_start(mblabel, expand = FALSE)
        fifobox.show()
        
	fstab.attach(self.diralloc, 0, 1, 0, 1, FILL, 0, 5)
	fstab.attach(self.fullchar, 0, 1, 1, 2, FILL, 0, 5)
	fstab.attach(self.allowdots, 0, 1, 2, 3, FILL, 0, 5)
	fstab.attach(self.followlinks, 0, 1, 3, 4, FILL, 0, 5)
        fstab.attach(self.periods, 0, 1, 4, 5, FILL, 0, 5)
        fstab.attach(fifobox, 2, 3, 0, 1, 0, 0, 0)
        fstab.attach(empty, 2, 3, 1, 5, 0, 0, 0)
        
	fstab.attach(self.joliet, 1, 2, 0, 1, FILL, 0, 5)
	fstab.attach(self.ratrock, 1, 2, 1, 2, FILL, 0, 5)
	fstab.attach(self.rockridge, 1, 2, 2, 3, FILL, 0, 5)
	fstab.attach(self.pad, 1, 2, 3, 4, FILL, 0, 5)
	fstab.attach(self.swab, 1, 2, 4, 5, FILL, 0, 5)

        fsframe.add(fstab)
        
        opttable.attach(fsframe, 0, 1, 0, 1, FILL, 0, 0, 0)

        # Recorder options
        recframe = GtkFrame(" Recorder / Reader")
        recframe.set_border_width(5)
        recframe.set_shadow_type(SHADOW_ETCHED_OUT)
        recframe.show()

        # Create a vertical box containing the SCSI-options, speed and mode checkboxes
	recordbox = GtkVBox(spacing = 10)
	recordbox.show()

        # Create a table containing the SCSI-options (Select CD-Writer)
	channel = GtkLabel("Channel: ")
        channel.set_alignment(1.0, 0.5)
        channel.show()
	id = GtkLabel("ID: ")
        id.set_alignment(1.0, 0.5)
        id.show()
	lun = GtkLabel("Lun: ")
        lun.set_alignment(1.0, 0.5)
	lun.show()

	self.channel = GtkEntry()
        self.channel.set_usize(35, -1)
	self.channel.show()
	self.id = GtkEntry()
        self.id.set_usize(35, -1)
	self.id.show()
	self.lun = GtkEntry()
        self.lun.set_usize(35, -1)
	self.lun.show()

	autobutton = GtkButton()
	box = self.XpmLabelBox(" Detect Recorder ")
	autobutton.add(box)
	autobutton.connect('clicked', self.DetectRecorder)
	autobutton.show()

	channelReader = GtkLabel("Channel: ")
        channelReader.set_alignment(1.0, 0.5)
        channelReader.show()
	idReader = GtkLabel("ID: ")
        idReader.set_alignment(1.0, 0.5)
        idReader.show()
	lunReader = GtkLabel("Lun: ")
        lunReader.set_alignment(1.0, 0.5)
	lunReader.show()

	self.channelReader = GtkEntry()
        self.channelReader.set_usize(35, -1)
	self.channelReader.show()
	self.idReader = GtkEntry()
        self.idReader.set_usize(35, -1)
	self.idReader.show()
	self.lunReader = GtkEntry()
        self.lunReader.set_usize(35, -1)
	self.lunReader.show()

	btnAutoReader = GtkButton()
	box = self.XpmLabelBox(" Detect Reader ")
	btnAutoReader .add(box)
	btnAutoReader .connect('clicked', self.DetectReader)
	btnAutoReader .show()

        speedlabel = GtkLabel("Speed (CD-W): ")
        speedlabel.set_alignment(1.0, 0.5)
        speedlabel.show()
        self.speed = GtkSpinButton(GtkAdjustment(8,2,64,2,10,0), 0, 0)
        self.speed.show()
        xlabel = GtkLabel("x")
        xlabel.show()

        speedtable = GtkTable(2, 3)
        speedtable.attach(speedlabel, 0, 1, 0, 1, EXPAND|FILL, 0, 5)
        speedtable.attach(self.speed, 1, 2, 0, 1, FILL, 0, 5)
        speedtable.attach(xlabel, 2, 3, 0, 1, 0, FILL, 5)

        speedlabel = GtkLabel("Speed (CD-RW): ")
        speedlabel.set_alignment(1.0, 0.5)
        speedlabel.show()
        self.speedRW = GtkSpinButton(GtkAdjustment(4,2,64,2,10,0), 0, 0)
        self.speedRW.show()
        xlabel = GtkLabel("x")
        xlabel.show()

        speedtable.attach(speedlabel, 0, 1, 1, 2, EXPAND|FILL, 0, 5)
        speedtable.attach(self.speedRW, 1, 2, 1, 2, FILL, 0, 5)
        speedtable.attach(xlabel, 2, 3, 1, 2, FILL, 0, 5)

        speedtable.show()

	recordtab = GtkTable(3, 7)
	recordtab.attach(channel, 0, 1, 0, 1, FILL, 0, 5)
	recordtab.attach(self.channel, 1, 2, 0, 1, 0, 0, 5)
	recordtab.attach(id, 0, 1, 1, 2, FILL, 0, 5)
	recordtab.attach(self.id, 1, 2, 1, 2, 0, 0, 5)
	recordtab.attach(lun, 0, 1, 2, 3, FILL, 0, 5)
	recordtab.attach(self.lun, 1, 2, 2, 3, 0, 0, 5)
	recordtab.attach(autobutton, 2, 3, 0, 3, SHRINK, 0, 5)

	recordtab.attach(channelReader, 3, 4, 0, 1, FILL, 0, 5)
	recordtab.attach(self.channelReader, 4, 5, 0, 1, 0, 0, 5)
	recordtab.attach(idReader, 3, 4, 1, 2, FILL, 0, 5)
	recordtab.attach(self.idReader, 4, 5, 1, 2, 0, 0, 5)
	recordtab.attach(lunReader, 3, 4, 2, 3, FILL, 0, 5)
	recordtab.attach(self.lunReader, 4, 5, 2, 3, 0, 0, 5)
	recordtab.attach(btnAutoReader, 5, 6, 0, 3, SHRINK, 0, 5)

        recordtab.attach(speedtable, 6, 7, 0, 3, 0, 0, 5)
	recordtab.show()

        # Create a table containing varions Recorder options
	recopttab = GtkTable(1, 2)
	recopttab.show()

	self.dummy = GtkCheckButton("Dummy operation")
	self.dummy.show()
	self.eject = GtkCheckButton("Eject CD after operation")
	self.eject.show()

	recopttab.attach(self.dummy, 0, 1, 0, 1, FILL, 0, 5)
	recopttab.attach(self.eject, 1, 2, 0, 1, FILL, 0, 5)

	recordbox.pack_start(recordtab, expand = FALSE, fill = FALSE)
	recordbox.pack_start(recopttab, expand = FALSE, fill = FALSE)

        recframe.add(recordbox)

        opttable.attach(recframe, 0, 1, 1, 2, FILL, 0, 0, 0)

        # Miscellaneous options
        miscframe = GtkFrame(" Miscellaneous ")
        miscframe.set_border_width(5)
        miscframe.set_shadow_type(SHADOW_ETCHED_OUT)
        miscframe.show()

        miscvbox = GtkVBox(spacing = 5)
        miscvbox.set_border_width(5)
        miscvbox.show()
        
        imagelabel = GtkLabel("Image path: ")
        imagelabel.show()

	self.imageentry = GnomeFileEntry("tempimage")
	self.imageentry.set_usize(350, -1)
        self.imageentry.show()

        imagebox = GtkHBox(spacing = 5)
        imagebox.pack_start(imagelabel, expand = FALSE)
        imagebox.pack_start(self.imageentry, expand = FALSE)
        imagebox.show()

        miscvbox.pack_start(imagebox, expand = FALSE)

        mischbox = GtkHBox(spacing = 5)
        mischbox.show()
        
	self.convmp3wav = GtkCheckButton("Automatically convert mp3-files to wav-files")
	self.convmp3wav.show()

        self.normalize = GtkCheckButton("Normalize wav-files")
        self.normalize.show()
        
        labPlayer = GtkLabel(" Audio Player:")
        labPlayer.show()

        optionplayer = GtkOptionMenu()
        self.playermenu = GtkMenu()
        self.playermenu.show()
        self.playerMpg123 = GtkRadioMenuItem(None, "mpg123")
        self.playerMpg123.show()
        self.playermenu.append(self.playerMpg123)
        self.playerXMMS = GtkRadioMenuItem(self.playerMpg123, "xmms")
        self.playerXMMS.show()
        self.playermenu.append(self.playerXMMS)
        optionplayer.set_menu(self.playermenu)
        optionplayer.show()
        
        mischbox.pack_start(self.convmp3wav, expand = FALSE)
        mischbox.pack_start(self.normalize, expand = FALSE)
        mischbox.pack_start(labPlayer, expand = FALSE)
        mischbox.pack_start(optionplayer, expand = FALSE)

        miscvbox.pack_start(mischbox, expand = FALSE)
        
        miscframe.add(miscvbox)

        opttable.attach(miscframe, 0, 1, 2, 3, FILL, 0, 0, 0)
        
        frame.add(opttable)

    # Save options to the config file
    def SaveOptions(self):

        filename = "%s/%s" % (environ["HOME"], deffile)
        file = open(filename, "w")

	# Filesystem options
	file.write("[Filesystem]\n")
	file.write("DisableDeepRelocation: %s\n" % beautify(self.diralloc.active))
	file.write("FullIso9660Filenames: %s\n" % beautify(self.fullchar.active))
	file.write("AllowLeadingDots: %s\n" % beautify(self.allowdots.active))
	file.write("FollowLinks: %s\n" % beautify(self.followlinks.active))
	file.write("Joliet: %s\n" % beautify(self.joliet.active))
	file.write("RationalRock: %s\n" % beautify(self.ratrock.active))
	file.write("Rock: %s\n" % beautify(self.rockridge.active))
	file.write("OmitPeriod: %s\n" % beautify(self.periods.active))
	file.write("Pad: %s\n" % beautify(self.pad.active))
	file.write("Swab: %s\n" % beautify(self.swab.active))

	# Recorder options
	file.write("\n[Recorder]\n")
        file.write("Speed: %d\n" % self.speed.get_value_as_int())
        file.write("SpeedRW: %d\n" % self.speedRW.get_value_as_int())
	file.write("Channel: %s\n" % self.channel.get_text())
        file.write("ID: %s\n" % self.id.get_text())
	file.write("LUN: %s\n" % self.lun.get_text())
	file.write("DummyMode: %s\n" % beautify(self.dummy.active))
	file.write("Eject: %s\n" % beautify(self.eject.active))

        # Reader options
	file.write("\n[Reader]\n")
	file.write("Channel: %s\n" % self.channelReader.get_text())
        file.write("ID: %s\n" % self.idReader.get_text())
	file.write("LUN: %s\n" % self.lunReader.get_text())
        
        # Temporary writing settings
        file.write("\n[Settings]\n")
        file.write("Mode: %d\n" % self.tw_mode)
        file.write("Fixate: %s\n" % beautify(self.tw_fixate))
        file.write("Multisession: %s\n" % beautify(self.tw_multisession))

	# Miscellaneous options
        file.write("\n[Miscellaneous]\n")
        file.write("FIFO: %d\n" % self.fifosize.get_value_as_int())
        file.write("ImagePath: %s\n" % self.imageentry.gtk_entry().get_text())
        file.write("ConvertMp3: %s\n" % beautify(self.convmp3wav.active))
        if self.playerMpg123.active:
            file.write("Mp3-Player: %s\n" % playerMpg123)
        elif self.playerXMMS.active:
            file.write("Mp3-Player: %s\n" % playerXMMS)
        file.write("Normalize: %s\n" % beautify(self.normalize.active))

        file.close()

    # Load configuration options or set defaults
    def LoadOptions(self):
        filename = "%s/%s" % (environ["HOME"], deffile)

        setDefault = False
        
	if os.path.exists(filename):                        # Load configuration file
            try:
                self.parser.read(filename)

                # Filesystem options
                self.diralloc.set_active(beautify(self.parser.get("Filesystem", "DisableDeepRelocation")))
                self.fullchar.set_active(beautify(self.parser.get("Filesystem", "FullIso9660Filenames")))
                self.allowdots.set_active(beautify(self.parser.get("Filesystem", "AllowLeadingDots")))
                self.followlinks.set_active(beautify(self.parser.get("Filesystem", "FollowLinks")))
                self.joliet.set_active(beautify(self.parser.get("Filesystem", "Joliet")))
                self.ratrock.set_active(beautify(self.parser.get("Filesystem", "RationalRock")))
                self.rockridge.set_active(beautify(self.parser.get("Filesystem", "Rock")))
                self.periods.set_active(beautify(self.parser.get("Filesystem","OmitPeriod")))
                self.pad.set_active(beautify(self.parser.get("Filesystem", "Pad")))
                self.swab.set_active(beautify(self.parser.get("Filesystem","Swab")))
	    
                # Recorder options
                self.speed.set_value(atoi(self.parser.get("Recorder", "Speed")))
                self.speedRW.set_value(atoi(self.parser.get("Recorder", "SpeedRW")))
                self.channel.set_text(self.parser.get("Recorder", "Channel"))
                self.id.set_text(self.parser.get("Recorder", "ID"))
                self.lun.set_text(self.parser.get("Recorder", "LUN"))
                self.dummy.set_active(beautify(self.parser.get("Recorder", "DummyMode")))
                self.eject.set_active(beautify(self.parser.get("Recorder", "Eject")))

                # Reader options
                self.channelReader.set_text(self.parser.get("Reader", "Channel"))
                self.idReader.set_text(self.parser.get("Reader", "ID"))
                self.lunReader.set_text(self.parser.get("Reader", "LUN"))

                # Temporary writing settings
                self.tw_mode = atoi(self.parser.get("Settings", "Mode"))
                self.tw_fixate = beautify(self.parser.get("Settings", "Fixate"))
                self.tw_multisession = beautify(self.parser.get("Settings", "Multisession"))
                
                # Miscellaneous options
                self.fifosize.set_value(atoi(self.parser.get("Miscellaneous", "FIFO")))
                tmpImagePath = self.parser.get("Miscellaneous", "ImagePath")
                self.imageentry.gtk_entry().set_text(tmpImagePath)
                self.readcdimage.set_default_path(tmpImagePath)
                self.cdimage.set_default_path(tmpImagePath)
                self.convmp3wav.set_active(beautify(self.parser.get("Miscellaneous", "ConvertMp3")))
                player = self.parser.get("Miscellaneous", "Mp3-Player")
                if player == playerMpg123:
                    self.playermenu.activate_item(self.playerMpg123, TRUE)
                elif player == playerXMMS:
                    self.playermenu.activate_item(self.playerXMMS, TRUE)
                self.normalize.set_active(beautify(self.parser.get("Miscellaneous", "Normalize")))
                
            except:
                
                setDefault = True

        else:                                               # Set default values

            setDefault = True

        if (setDefault == True):
            sys.stdout.write("Warning: couldn't read settings - setting default values !\n\n")
            
	    # Filesystem options
	    self.diralloc.set_active(1)
	    self.fullchar.set_active(1)
	    self.allowdots.set_active(1)
	    self.followlinks.set_active(0)
	    self.joliet.set_active(1)
	    self.ratrock.set_active(0)
	    self.rockridge.set_active(1)
	    self.periods.set_active(0)
	    self.pad.set_active(1)
	    self.swab.set_active(1)
	    
	    # Recorder options
            self.speed.set_value(8)
            self.speedRW.set_value(4)
	    self.channel.set_text("00")
	    self.id.set_text("00")
	    self.lun.set_text("00")
	    self.dummy.set_active(0)
	    self.eject.set_active(1)

            # Reader options
            self.channelReader.set_text("00")
            self.idReader.set_text("00")
            self.lunReader.set_text("00")

            # Temporary writing settings
            self.tw_mode = 0
            self.tw_fixate = 1
            self.tw_multisession = 0

	    # Miscellaneous options
            self.fifosize.set_value(4)
            self.convmp3wav.set_active(1)
            self.playermenu.activate_item(self.playerXMMS, TRUE)
            self.normalize.set_active(0)

    # Detect SCSI-recorder
    def DetectRecorder(self, args):
        self.DetectSCSIDevice(deviceRecorder)

    # Detect SCSI-reader
    def DetectReader(self, args):
        self.DetectSCSIDevice(deviceReader)
    
    # Detect SCSI-devices
    def DetectSCSIDevice(self, device):
        pipe = popen2.popen2("cdrecord -scanbus 2>&1")
	output = pipe[0].readlines()
        devicelist = []
	for i in range(len(output)):
	    line = output[i]
            if (line[0] == "\t") and (line[1] in digits):
                if find(line, "CD-ROM") != -1:
                    channel = line[1]
                    id = line[3]
                    lun = line[5]
                    vendor = "%s %s %s" % (strip(line[13:21]), strip(line[24:40]), strip(line[43:47]))
                    devicelist.append([channel, id, lun, vendor])
        pipe[0].close()
        pipe[1].close()
        pipe = None
        if len(devicelist) == 0:
            GnomeErrorDialog("Couldn't detect your reader / recorder !\nPlease make sure that you are root or that you're allowed to read /dev/sg* !")
        elif len(devicelist) == 1:
            if device == deviceReader:
                self.idReader.set_text(devicelist[0][1])
                self.channelReader.set_text(devicelist[0][0])
                self.lunReader.set_text(devicelist[0][2])
            elif device == deviceRecorder:
                self.id.set_text(devicelist[0][1])
                self.channel.set_text(devicelist[0][0])
                self.lun.set_text(devicelist[0][2])
        else:
            self.SelectSCSIDevice(device, devicelist)

    # Select the device if >1 devices found
    def SelectSCSIDevice(self, device, devicelist):
        self.scsiwindow = GnomeApp("ERoasterSCSI", "SCSI Devices")
        self.scsiwindow.set_border_width(1)
        self.scsiwindow.show()

        scsibox = GtkVBox(spacing = 5)
        scsibox.set_border_width(5)
        scsibox.show()
        
        scsifrm = GtkFrame(" Select SCSI-device ")
	scsifrm.set_shadow_type(SHADOW_ETCHED_OUT)
        scsifrm.show()

        box = GtkVBox(spacing = 5)
        box.set_border_width(5)
        box.show()
        
        self.scsidevice = []
        for intCounter in range(len(devicelist)):
            if intCounter == 0:
                self.scsidevice.append(GtkRadioButton(None, devicelist[intCounter][3]))
            else:
                self.scsidevice.append(GtkRadioButton(self.scsidevice[intCounter - 1], devicelist[intCounter][3]))
            self.scsidevice[intCounter].show()
            box.pack_start(self.scsidevice[intCounter], expand = FALSE)

        scsifrm.add(box)

        btnSelect = GtkButton(" Select ")
        btnSelect.connect('clicked', self.SelectSCSIDeviceSelect)
        btnSelect.show()

        self.device = device
        self.devicelist = devicelist
        
        scsibox.pack_start(scsifrm, expand = FALSE)
        scsibox.pack_start(btnSelect, expand = FALSE)

        self.scsiwindow.set_contents(scsibox)

    # Select SCSI Device (Select button clicked)
    def SelectSCSIDeviceSelect(self, args):

        for intCounter in range(len(self.scsidevice)):
            if self.scsidevice[intCounter].active == 1:
                if self.device == deviceReader:
                    self.idReader.set_text(self.devicelist[intCounter][1])
                    self.channelReader.set_text(self.devicelist[intCounter][0])
                    self.lunReader.set_text(self.devicelist[intCounter][2])
                elif self.device == deviceRecorder:
                    self.id.set_text(self.devicelist[intCounter][1])
                    self.channel.set_text(self.devicelist[intCounter][0])
                    self.lun.set_text(self.devicelist[intCounter][2])

        self.scsiwindow.destroy()
        self.scsiwindow = None
        
    # Swap two entries in the Songlist
    def SongListSwap(self, row):
        sourceindex = self.songlist.get_text(self.songlist.focus_row, 0)
        targetindex = self.songlist.get_text(row, 0)
        self.songlist.set_text(self.songlist.focus_row, 0, targetindex)
        self.songlist.set_text(row, 0, sourceindex)
        self.songlist.row_move(self.songlist.focus_row, row)
        
    # When button is pressed move the selected file
    def SongListButtonPressed(self, widget, event=None):
        try:
            row = self.songlist.get_selection_info(event.x, event.y)[0]
        except:
            row = -1

    # Stop moving the file when the mouse button is released
    def SongListButtonReleased(self, widget, event=None):
        try:
            row = self.songlist.get_selection_info(event.x, event.y)[0]
        except:
            row = -1
        if (row != self.songlist.focus_row) and (row != -1) and (self.songlist.focus_row != -1):
            self.SongListSwap(row)
            
    # Move files up & down when "dragging"
    def SongListMotion(self, widget, event=None):
        try:
            row = self.songlist.get_selection_info(event.x, event.y)[0]
        except:
            row = -1
        if (row != self.songlist.focus_row) and (row != -1) and (self.songlist.focus_row != -1):
            self.SongListSwap(row)

    # Drag & Drop - Add file to the Songlist window
    def SongListDragReceive(self, window, context, x, y, data, info, time):
        filenames = split(data.data,"\n")
        for i in range(len(filenames)):
            basename = strip(replace(os.path.basename(filenames[i][5:]), '\000', ''))
            if (filenames[i][:5] == "file:") and (basename != "") and (basename != ".") and (basename != ".."):
                self.AddFileToList(filenames[i][5:])
        
    # Burn Audio CDs
    def BurnAudio(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.show()

        targets = [('text/plain', 0, -1)]

        # Volume ID
        volumeframe = GtkFrame(" Volume ID ")
        volumeframe.set_border_width(5)
        volumeframe.set_shadow_type(SHADOW_ETCHED_OUT)
        volumeframe.show()

        # Create a table containing the Volume ID
        volumetab = GtkTable(1, 2)
        volumetab.show()
        
	label = GtkLabel("Enter Volume ID: ")
	label.show()

	self.volumeidaudio = GtkEntry()
	self.volumeidaudio.show()

        volumetab.attach(label, 0, 1, 0, 1, FILL, 0, 5, 5)
        volumetab.attach(self.volumeidaudio, 1, 2, 0, 1, FILL, 0, 5, 5)

        volumeframe.add(volumetab)

        burnbox.pack_start(volumeframe, expand = FALSE)

        # Create a table containing the wav files
        wavframe = GtkFrame(" Wav / Mp3 files ")
        wavframe.set_border_width(5)
        wavframe.set_shadow_type(SHADOW_ETCHED_OUT)
        wavframe.show()

        self.songlist = GtkCList(3, [ " Nr ", " Filename ", " Time " ])
        self.songlist.show()
        self.songlist.connect("drag_data_received", self.SongListDragReceive)
        self.songlist.drag_dest_set(DEST_DEFAULT_ALL, targets, GDK.ACTION_COPY)
        self.songlist.connect("button_press_event", self.SongListButtonPressed)
        self.songlist.connect("button_release_event", self.SongListButtonReleased)
        self.songlist.connect("motion_notify_event", self.SongListMotion)
        self.songlist.set_selection_mode(SELECTION_MULTIPLE)
        self.songlist.set_column_width(1, 385)
        
        self.songlist.set_column_justification(2, JUSTIFY_RIGHT)
        
        listwin = GtkScrolledWindow()
        listwin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        listwin.add(self.songlist)
        listwin.show()
        
        addbutton = GtkButton()
	box = self.XpmLabelBox(" Add File(s) ")
        box.set_usize(95, -1)
	addbutton.add(box)
	addbutton.connect('clicked', self.AddFile)
	addbutton.show()

        delbutton = GtkButton()
	box = self.XpmLabelBox(" Remove File(s) ")
        box.set_usize(95, -1)
	delbutton.add(box)
	delbutton.connect('clicked', self.DelFile)
	delbutton.show()

        btnPlay = GtkButton()
        box = self.XpmLabelBox(" Play File ")
        box.set_usize(95, -1)
        btnPlay.add(box)
        btnPlay.connect('clicked', self.PlayAudioFile)
        btnPlay.show()
        
        sizelabel = GtkLabel("Total time: ")
        sizelabel.set_alignment(1.0, 0.5)
        sizelabel.show()

        self.audiosize = GtkEntry()
        self.audiosize.set_editable(FALSE)
        self.audiosize.set_usize(40, -1)
        self.audiosize.set_text("00:00")
        self.audiosize.show()
        
        emptyLabel = GtkLabel()
        emptyLabel.show()

        wavtable = GtkTable(5, 3)
        wavtable.show()

        wavtable.attach(listwin, 0, 1, 0, 5, EXPAND|FILL, EXPAND|FILL, 5, 5)
        wavtable.attach(addbutton, 1, 3, 0, 1, 0, 0, 5, 5)
        wavtable.attach(delbutton, 1, 3, 1, 2, 0, 0, 5, 5)
        wavtable.attach(emptyLabel, 1, 3, 2, 3, 0, EXPAND|FILL, 5, 5)
        wavtable.attach(btnPlay, 1, 3, 3, 4, 0, 0, 5, 5)
        wavtable.attach(sizelabel, 1, 2, 4, 5, FILL, 0, 5, 5)
        wavtable.attach(self.audiosize, 2, 3, 4, 5, FILL, 0, 5, 5)
        wavframe.add(wavtable)
        
        burnbox.pack_start(wavframe)
        
	frame.add(burnbox)

    # Play an audio file using xmms
    def PlayAudioFile(self, args):
        if len(self.songlist.selection) == 0:
            GnomeErrorDialog("No audio file selected for playing")
            return

        if self.playerXMMS.active:
            command = "xmms"
        elif self.playerMpg123.active:
            command = "mpg123"
            
        for i in self.songlist.selection:
            intIndex = self.songlist.selection[i]
            command = "%s \"%s\"" % (command, self.songlist.get_text(intIndex, 1))

        system(command)
        
    # Create frame for special operations on CDs
    def SpecialOperation(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.set_border_width(5)
        burnbox.show()

        blankfrm = GtkFrame(" Select operation ")
	blankfrm.set_shadow_type(SHADOW_ETCHED_OUT)
        blankfrm.show()

        self.blankDisc = GtkRadioButton(None, "Blank disc")
	self.blankDisc.show()
	self.blankMinimal = GtkRadioButton(self.blankDisc, "Blank minimal (PMA, TOC, pregap)")
	self.blankMinimal.show()
	self.blankSession = GtkRadioButton(self.blankMinimal, "Blank last session")
	self.blankSession.show()
        self.fixate = GtkRadioButton(self.blankSession, "Fixate CD")
	self.fixate.show()
	self.writeTOC = GtkRadioButton(self.fixate, "Write TOC")
        self.writeTOC.set_sensitive(FALSE)
	self.writeTOC.show()

        blankTable = GtkTable(3,2)
        blankTable.attach(self.blankDisc, 0, 1, 0, 1, FILL, 0, 5)
        blankTable.attach(self.blankMinimal, 0, 1, 1, 2, FILL, 0, 5)
        blankTable.attach(self.blankSession, 0, 1, 2, 3, FILL, 0, 5)
        blankTable.attach(self.fixate, 1, 2, 0, 1, FILL, 0, 5)
        blankTable.attach(self.writeTOC, 1, 2, 1, 2, FILL, 0, 5)
        blankTable.show()

        blankfrm.add(blankTable)

        miscfrm = GtkFrame()
        miscfrm.set_shadow_type(SHADOW_ETCHED_OUT)
        miscfrm.show()

        blankBtn = GtkButton()
	box = self.XpmLabelBox(" Perform special operation ", "%s/burn.xpm" % iconpath)
	blankBtn.add(box)
        blankBtn.connect('clicked', self.PerformSpecialOperation)
        blankBtn.show()

        btnbox = GtkHBox(spacing = 5)
        btnbox.set_border_width(5)
        btnbox.show()

        leftlabel = GtkLabel()
        leftlabel.show()

        rightlabel = GtkLabel()
        rightlabel.show()
        
        btnbox.pack_start(leftlabel)
        btnbox.pack_start(blankBtn, expand = FALSE)
        btnbox.pack_start(rightlabel)
        
        miscfrm.add(btnbox)
        
        burnbox.pack_start(blankfrm, expand = FALSE)
        burnbox.pack_start(miscfrm, expand = FALSE)
        
	frame.add(burnbox)

    # Really perform special operation on CD now
    def PerformSpecialOperation(self, args):
        if self.blankDisc.active == 1:
            self.writermode = BlankCDAll
        elif self.blankMinimal.active == 1:
            self.writermode = BlankCDMinimal
        elif self.blankSession.active == 1:
            self.writermode = BlankCDLast
        elif self.fixate.active == 1:
            self.writermode = modeFixateCD
        elif self.writeTOC.active == 1:
            return

        self.BurnCDNow()
        
    # Delete selected files from the list of wavefiles
    def DelFile(self, args):
        if (len(self.songlist.selection) == 0):
            GnomeErrorDialog("You have to select files first!")
        else:
            list = copy.copy(self.songlist.selection)
            list.sort()
            list.reverse()
            for i in range(len(list)):
                filename = self.songlist.get_text(list[i], 1)
                self.totalaudiosize = self.totalaudiosize - self.tracksize[filename]
                self.totaltracklength = self.totaltracklength - self.tracklength[filename]
                self.audiosize.set_text(nicetime(self.totaltracklength))
                
                del self.tracksize[filename]
                del self.tracklength[filename]
                self.songlist.remove(list[i])

                # Renumber files
                line = list[i]
                try:
                    index = self.songlist.get_text(line, 0)
                except:
                    index = None
                while (index != None):
                    self.songlist.set_text(line, 0, str(int(self.songlist.get_text(line, 0)) - 1))
                    line = line + 1
                    try:
                        index = self.songlist.get_text(line, 0)
                    except:
                        index = None

    # Actually add the file to the list
    def AddFileToList(self, args):
        if (type(args) == StringType):
            filename = strip(args)
        else:
            filename = strip(self.addfilewin.get_filename())

        found = False
        line = 0
        try:
            listname = self.songlist.get_text(line, 1)
        except:
            listname = None
        while (listname != None):
            if (listname == filename):
                found = True
            line = line + 1
            try:
                listname = self.songlist.get_text(line, 1)
            except:
                listname = None

        if (found == False):
            filesize = os.path.getsize(filename)
            self.totalaudiosize = self.totalaudiosize + filesize
            seconds = self.GetAudioTrackLength(filename)
            self.tracksize[filename] = filesize
            self.tracklength[filename] = seconds
            self.totaltracklength = self.totaltracklength + seconds
            playtime = nicetime(seconds)
            self.songlist.append([str(line + 1), filename, playtime])
            self.audiosize.set_text(nicetime(self.totaltracklength))

    # Converts a mp3-file to a wav-file
    def ConvertMp3ToWav(self):

        if self.convmp3wav.active:
            
            imagepath = self.imageentry.gtk_entry().get_text()
            if imagepath == "":
                GnomeErrorDialog("You have to set a image path first !")
                return

            win = GnomeApp("ERoasterPleaseWait", "Please Wait")
            win.set_border_width(5)
            win.show()

            label = GtkLabel(" Converting wav-files to mp3-files.\n This may take a minute ... ")
            label.show()

            waitframe = GtkFrame(" Please Wait ")
            waitframe.set_border_width(5)
            waitframe.set_shadow_type(SHADOW_ETCHED_OUT)
            waitframe.add(label)
            waitframe.show()

            vbox = GtkVBox(spacing = 5)
            vbox.set_border_width(0)
            vbox.pack_start(waitframe)
            vbox.show()

            win.set_contents(vbox)

            # FIXME: make sure window is being drawn
            while events_pending():
                mainiteration()

            line = 0
            try:
                filename = self.songlist.get_text(line, 1)
            except:
                filename = None
            while (filename != None):

                if (lower(filename[-4:]) == ".mp3"):
                    tracklength = self.songlist.get_text(line, 2)
                    if imagepath[-1] == "/":
                        wavfilename = "%s%s.wav" % (imagepath, os.path.basename(filename)[:-4])
                    else:
                        wavfilename = "%s/%s.wav" % (imagepath, os.path.basename(filename)[:-4])

                    command = "(mpg123 --stereo -s \"%s\" | sox -t raw -r 44100  -w -s -c 2 - -t wav \"%s\" ) > /dev/null 2>&1" % (filename, wavfilename)
                        
                    system(command)
                    self.songlist.remove(line)
                    self.songlist.insert(line, [str(line + 1), wavfilename, tracklength])

                    # Correct filesize
                    self.totalaudiosize = self.totalaudiosize - self.tracksize[filename]
                    filesize = os.path.getsize(wavfilename)
                    self.tracksize[wavfilename] = filesize
                    self.totalaudiosize = self.totalaudiosize + self.tracksize[wavfilename]
                    
                line = line + 1
                try:
                    filename = self.songlist.get_text(line, 1)
                except:
                    filename = None

            win.destroy()
            win = None
        
    # Open the file window to select a list of wavefiles
    def AddFile(self, args):
        self.addfilewin = GtkFileSelection("Select wave files")
        self.addfilewin.ok_button.connect("clicked", self.AddFileToList)
        self.addfilewin.cancel_button.connect("clicked", self.addfilewin.destroy)
        self.addfilewin.show()

    # Drag & Drop - Add file to the datafile window
    def DataDragReceive(self, window, context, x, y, data, info, time):
        filenames = split(data.data,"\n")
        for i in range(len(filenames)):
            basename = strip(replace(os.path.basename(filenames[i][5:]), '\000', ''))
            if (filenames[i][:5] == "file:") and (basename != "") and (basename != ".") and (basename != ".."):
                self.AddDataFileToList(filenames[i][5:])

    # Add Files to the Datafile list
    def AddDataFileToList(self, args):
        if (type(args) == StringType):
            filename = strip(args)
        else:
            filename = strip(self.adddatawin.get_filename())

        found = False
        line = 0
        try:
            listname = self.datafiles.get_text(line, 0)
        except:
            listname = None
        while (listname != None):
            if (listname == filename):
                found = True
                listname = None
            line = line + 1
            try:
                listname = self.datafiles.get_text(line, 0)
            except:
                listname = None

        if (found == False):
            if filename[-1] == "\000":
                filename = filename[:-1]
            if os.path.exists(filename):
                try:
                    size = du(filename) / 1024
                    self.totaldatasize = self.totaldatasize + size
                    self.datafilesize[filename] = size
                    self.datafiles.append([filename, "%.0f KB" % size])
                    self.datasize.set_text("%.0f MB" % (self.totaldatasize / 1024))
                except:
                    GnomeErrorDialog("Warning: Can't stat \"%s\"!" % filename)
            else:
                GnomeErrorDialog("Warning: \"%s\" doesn't exist!" % filename)
        
    # Open the file window to select a list of wavefiles
    def AddDataFile(self, args):
        self.adddatawin = GtkFileSelection("Select files / directories")
        self.adddatawin.ok_button.connect("clicked", self.AddDataFileToList)
        self.adddatawin.cancel_button.connect("clicked", self.adddatawin.destroy)
        self.adddatawin.show()

    # Delete files from the data file selection
    def DelDataFile(self, args):
        if (len(self.datafiles.selection) == 0):
            GnomeErrorDialog("You have to select files/directories first!")
        else:
            list = copy.copy(self.datafiles.selection)
            list.sort()
            list.reverse()
            for i in range(len(list)):
                filename = self.datafiles.get_text(list[i], 0)
                size = self.datafilesize[filename]
                self.totaldatasize = self.totaldatasize - size
                self.datasize.set_text("%d MB" % (self.totaldatasize / 1024))
                del self.datafilesize[filename]
                self.datafiles.remove(list[i])

    # Select a data row
    def DataSelectRow(self, clist, row, column, event):
        size = atoi(self.datafiles.get_text(row, 1)[:-3])
        self.selecteddatasize = self.selecteddatasize + size
        self.datasizesel.set_text("%d MB" % round(float(self.selecteddatasize) / 1024))

    # Unselect a data row
    def DataUnselectRow(self, clist, row, column, event):
        size = atoi(self.datafiles.get_text(row, 1)[:-3])
        self.selecteddatasize = self.selecteddatasize - size
        self.datasizesel.set_text("%d MB" % round(float(self.selecteddatasize) / 1024))

    # Calculate & select optimal directories
    def CalculateOptimize(self, args):
        maxsize = 0
        self.datafiles.unselect_all()
        if (self.cdsize650.active):
            maxsize = 650 * 1024
        elif (self.cdsize700.active):
            maxsize = 700 * 1024
        if (self.totaldatasize <= maxsize):
            self.datafiles.select_all()
        else:

            # Insert size & filename into a valuable data structure
            sizefilename = {}
            sizelist = []
            line = 0
            try:
                name = self.datafiles.get_text(line, 0)
                filename = self.datafiles.get_text(line, 0)
                size = self.datafilesize[filename]
            except:
                name = None
            while (name != None):
                if sizefilename.has_key(size):
                    sizefilename[size].append(name)
                else:
                    sizefilename[size] = [name]
                    sizelist.append(size)
                line = line + 1
                try:
                    name = self.datafiles.get_text(line, 0)
                    size = atoi(self.datafiles.get_text(line, 1)[:-3])
                except:
                    name = None
            sizelist.sort()
            sizelist.reverse()

            # Move through all available sizes
            selectedsize = 0
            selectedfilenames = []
            for i in range(len(sizelist)):
                size = sizelist[i]
                filenames = sizefilename[size]
                for j in range(len(filenames)):
                    filename = filenames[j]
                    if ((selectedsize + size) <= maxsize):
                        selectedsize = selectedsize + size
                        selectedfilenames.append(filename)

            # Select files
            line = 0
            try:
                name = self.datafiles.get_text(line, 0)
            except:
                name = None
            while (name != None):
                if (name in selectedfilenames):
                    self.datafiles.select_row(line, 0)
                line = line + 1
                try:
                    name = self.datafiles.get_text(line, 0)
                except:
                    name = None
            
    # Burn Data CDs
    def BurnData(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.show()

        targets = [('text/plain', 0, -1)]

        # Volume ID
        volumeframe = GtkFrame(" Volume ID ")
        volumeframe.set_border_width(5)
        volumeframe.set_shadow_type(SHADOW_ETCHED_OUT)
        volumeframe.show()

        # Create a table containing the Volume ID
        volumetab = GtkTable(1, 2)
        volumetab.show()
        
	label = GtkLabel("Enter Volume ID: ")
	label.show()

	self.volumeid = GtkEntry()
	self.volumeid.show()

        volumetab.attach(label, 0, 1, 0, 1, FILL, 0, 5, 5)
        volumetab.attach(self.volumeid, 1, 2, 0, 1, FILL, 0, 5, 5)

        volumeframe.add(volumetab)

        burnbox.pack_start(volumeframe, expand = FALSE)

        # Create some fields for selecting the Source Directory & calculating the directory size

        sourceframe = GtkFrame(" Directories ")
        sourceframe.set_border_width(5)
        sourceframe.set_shadow_type(SHADOW_ETCHED_OUT)
        sourceframe.show()

        audiotable = GtkTable(8, 3)
        audiotable.show()
        
        self.datafiles = GtkCList(2, [ " Files / Directories ", " Size "])
        self.datafiles.show()
        self.datafiles.connect("select_row", self.DataSelectRow)
        self.datafiles.connect("unselect_row", self.DataUnselectRow)
        self.datafiles.connect("drag_data_received", self.DataDragReceive)
        self.datafiles.drag_dest_set(DEST_DEFAULT_ALL, targets, GDK.ACTION_COPY)
        self.datafiles.set_selection_mode(SELECTION_MULTIPLE)
        self.datafiles.set_column_width(0, 360)
        self.datafiles.set_column_justification(1, JUSTIFY_RIGHT)
        self.datafiles.set_auto_sort(TRUE)

        listwin = GtkScrolledWindow()
        listwin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        listwin.add(self.datafiles)
        listwin.show()

        adddirbtn = GtkButton()
        box = self.XpmLabelBox(" Add File(s) ")
        box.set_usize(105, -1)
	adddirbtn.add(box)
	adddirbtn.connect('clicked', self.AddDataFile)
	adddirbtn.show()

        deldirbtn = GtkButton()
	box = self.XpmLabelBox(" Remove File(s) ")
        box.set_usize(105, -1)
	deldirbtn.add(box)
        deldirbtn.connect('clicked', self.DelDataFile)
        deldirbtn.show()

        calcbtn = GtkButton()
	box = self.XpmLabelBox(" AutoSelect ")
        box.set_usize(105, -1)
	calcbtn.add(box)
        calcbtn.connect('clicked', self.CalculateOptimize)
        calcbtn.show()

        emptyLabel = GtkLabel()
        emptyLabel.show()

        sizelabel = GtkLabel("Total: ")
        sizelabel.set_alignment(1.0, 0.5)
        sizelabel.show()

        self.datasize = GtkEntry()
        self.datasize.set_editable(FALSE)
        self.datasize.set_usize(60, -1)
        self.datasize.set_text("0 MB")
        self.datasize.show()

        selectsize = GtkLabel("Selected: ")
        selectsize.set_alignment(1.0, 0.5)
        selectsize.show()

        self.datasizesel = GtkEntry()
        self.datasizesel.set_editable(FALSE)
        self.datasizesel.set_usize(60, -1)
        self.datasizesel.set_text("0 MB")
        self.datasizesel.show()

        self.cdsize650 = GtkRadioButton(None, "650 MB")
	self.cdsize650.show()
	self.cdsize700 = GtkRadioButton(self.cdsize650, "700 MB")
	self.cdsize700.show()
        
        audiotable.attach(listwin, 0, 1, 0, 7, EXPAND|FILL, EXPAND|FILL, 5, 5)
        audiotable.attach(adddirbtn, 1, 3, 0, 1, 0, 0, 5, 5)
        audiotable.attach(deldirbtn, 1, 3, 1, 2, 0, 0, 5, 5)
        audiotable.attach(calcbtn, 1, 3, 2, 3, 0, 0, 5, 5)
        audiotable.attach(emptyLabel, 1, 3, 3, 4, 0, EXPAND|FILL, 5)
        audiotable.attach(self.cdsize650, 1, 2, 4, 5, 0, 0)
        audiotable.attach(self.cdsize700, 2, 3, 4, 5, 0, 0)
        audiotable.attach(sizelabel, 1, 2, 5, 6, FILL, 0)
        audiotable.attach(self.datasize, 2, 3, 5, 6, 0, 0, 0, 5)
        audiotable.attach(selectsize, 1, 2, 6, 7, FILL, 0)
        audiotable.attach(self.datasizesel, 2, 3, 6, 7, 0, 0, 5, 5)
        
        sourceframe.add(audiotable)
        
        burnbox.pack_start(sourceframe)

	frame.add(burnbox)

    # Calculate the size of a directory
    def CalculateSize(self, args):
        directory = self.dirfield.gtk_entry().get_text()
        if (strip(directory) == ""):
            GnomeErrorDialog("Please select a directory first!")
        else:
            pipe = popen("du -s -m \"%s\"" % directory)
            output = striplist(pipe.readlines())
            pipe.close()
            directorysize = split(output[0])[0]
            self.sizefield.set_text("%s MB" % directorysize)
        
    # Close the main window (save options first)
    def Destroy(self, args):
        self.SaveOptions()
	self.window.destroy()
	mainquit()

    # Get the information of the image
    def ImageInfo(self, args):
        image = self.cdimage.gtk_entry().get_text()
        if (image == ""):
            GnomeErrorDialog("Please select a image file first.")
            return
        if (not os.path.exists(image)) or (os.path.isdir(image)):
            GnomeErrorDialog("The image file \"%s\" is not valid." % image)
            return
        pipe = popen2.popen2("isoinfo -d -i \"%s\"" % image)
        output = striplist(pipe[0].readlines())
        for i in range(len(output)):
            line = output[i]
            if (line[:10] == "System id:"): self.sysidtext.set_text(line[11:])
            elif (line[:10] == "Volume id:"): self.volidtext.set_text(line[11:])
            elif (line[:14] == "Volume set id:"): self.volsettext.set_text(line[15:])
            elif (line[:13] == "Publisher id:"): self.pubidtext.set_text(line[14:])
            elif (line[:15] == "Application id:"): self.appidtext.set_text(line[16:])
            elif (line[:22] == "Logical block size is:"): self.blsizetext.set_text(line[23:])
            elif (line[:15] == "Volume size is:"): self.volsizetext.set_text(line[16:])
        pipe[0].close()
        pipe[1].close()

    # Write a Image
    def BurnImage(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.show()

        # Select Image
        imageframe = GtkFrame(" ISO Image ")
        imageframe.set_border_width(5)
        imageframe.set_shadow_type(SHADOW_ETCHED_OUT)
        imageframe.show()

        # Create a table containing the Image selection

        imagetab = GtkTable(1, 3)
        imagetab.show()
        
	label = GtkLabel("Select CD Image: ")
	label.show()

	self.cdimage = GnomeFileEntry("imageentry")
	self.cdimage.set_usize(350, -1)
	self.cdimage.show()

        infobutton = GtkButton()
	infobox = self.XpmLabelBox(" Image info ")
	infobutton.add(infobox)
	infobutton.connect('clicked', self.ImageInfo)
	infobutton.show()
        
        imagetab.attach(label, 0, 1, 0, 1, FILL, 0, 5, 5)
        imagetab.attach(self.cdimage, 1, 2, 0, 1, FILL, 0, 5, 5)
        imagetab.attach(infobutton, 2, 3, 0, 1, FILL, 0, 5, 5)
        
        imageframe.add(imagetab)
        
        burnbox.pack_start(imageframe, expand = FALSE)

        # Frame which contains the iso information

        infoframe = GtkFrame(" ISO Information ")
        infoframe.set_border_width(5)
        infoframe.set_shadow_type(SHADOW_ETCHED_OUT)
        infoframe.show()

        infotab = GtkTable(5, 5)
        infotab.show()

        sysidlabel = GtkLabel("System ID:")
        sysidlabel.set_alignment(1.0, 0.5)
        sysidlabel.show()

        self.sysidtext = GtkEntry()
        self.sysidtext.set_usize(130, -1)
        self.sysidtext.set_editable(FALSE)
        self.sysidtext.show()

        infotab.attach(sysidlabel, 0, 1, 0, 1, FILL, 0, 5, 5)
        infotab.attach(self.sysidtext, 1, 2, 0, 1, FILL, 0, 5, 5)

        volidlabel = GtkLabel("Volume ID:")
        volidlabel.set_alignment(1.0, 0.5)
        volidlabel.show()

        self.volidtext = GtkEntry()
        self.volidtext.set_usize(130, -1)
        self.volidtext.set_editable(FALSE)
        self.volidtext.show()

        infotab.attach(volidlabel, 2, 3, 0, 1, FILL, 0, 5)
        infotab.attach(self.volidtext, 3, 4, 0, 1, FILL, 0, 5)

        volsetlabel = GtkLabel("Volume set ID:")
        volsetlabel.set_alignment(1.0, 0.5)
        volsetlabel.show()

        self.volsettext = GtkEntry()
        self.volsettext.set_usize(250, -1)
        self.volsettext.set_editable(FALSE)
        self.volsettext.show()

        infotab.attach(volsetlabel, 0, 1, 1, 2, FILL, 0, 5)
        infotab.attach(self.volsettext, 1, 4, 1, 2, FILL, 0, 5)
        
        pubidlabel = GtkLabel("Publisher ID:")
        pubidlabel.set_alignment(1.0, 0.5)
        pubidlabel.show()

        self.pubidtext = GtkEntry()
        self.pubidtext.set_usize(250, -1)
        self.pubidtext.set_editable(FALSE)
        self.pubidtext.show()

        infotab.attach(pubidlabel, 0, 1, 2, 3, FILL, 0, 5, 5)
        infotab.attach(self.pubidtext, 1, 4, 2, 3, FILL, 0, 5, 5)
        
        appidlabel = GtkLabel("Application ID:")
        appidlabel.set_alignment(1.0, 0.5)
        appidlabel.show()

        self.appidtext = GtkEntry()
        self.appidtext.set_usize(350, -1)
        self.appidtext.set_editable(FALSE)
        self.appidtext.show()

        infotab.attach(appidlabel, 0, 1, 3, 4, FILL, 0, 5)
        infotab.attach(self.appidtext, 1, 4, 3, 4, FILL, 0, 5)

        blsizelabel = GtkLabel("Logical block size:")
        blsizelabel.set_alignment(1.0, 0.5)
        blsizelabel.show()

        self.blsizetext = GtkEntry()
        self.blsizetext.set_usize(80, -1)
        self.blsizetext.set_editable(FALSE)
        self.blsizetext.show()

        emptylabel = GtkLabel()
        emptylabel.show()
        
        infotab.attach(blsizelabel, 0, 1, 4, 5, FILL, 0, 5, 5)
        infotab.attach(self.blsizetext, 1, 2, 4, 5, FILL, 0, 5, 5)
        
        volsizelabel = GtkLabel("Volume size:")
        volsizelabel.set_alignment(1.0, 0.5)
        volsizelabel.show()

        self.volsizetext = GtkEntry()
        self.volsizetext.set_usize(180, -1)
        self.volsizetext.set_editable(FALSE)
        self.volsizetext.show()

        infotab.attach(volsizelabel, 2, 3, 4, 5, FILL, 0, 5)
        infotab.attach(self.volsizetext, 3, 4, 4, 5, FILL, 0, 5)

        infoframe.add(infotab)
        
        burnbox.pack_start(infoframe, expand = FALSE)

	frame.add(burnbox)

    # Create a Frame for reading CDs
    
    def ReadCDFrame(self, frame):
        readCDbox = GtkVBox(spacing = 5)
        readCDbox.show()

        # Frame containing a button for reading a image

        readlabel = GtkLabel("Image filename: ")
        readlabel.show()
        
        readframe = GtkFrame(" Read ISO Image ")
        readframe.set_border_width(5)
        readframe.set_shadow_type(SHADOW_ETCHED_OUT)
        readframe.show()

        readtab = GtkTable(1, 3)
        readtab.show()

	self.readcdimage = GnomeFileEntry("readimageentry")
	self.readcdimage.set_usize(350, -1)
	self.readcdimage.show()

        readbutton = GtkButton()
        readbox = self.XpmLabelBox(" Read Image ")
        readbutton.add(readbox)
        readbutton.connect('clicked', self.ReadImage)
        readbutton.show()

        readtab.attach(readlabel, 0, 1, 0, 1, FILL, 0, 5, 5)
        readtab.attach(self.readcdimage, 1, 2, 0, 1, FILL, 0, 5, 5)
        readtab.attach(readbutton, 2, 3, 0, 1, FILL, 0, 5, 5)

        readframe.add(readtab)
        
        readCDbox.pack_start(readframe, expand = FALSE)

        readaudioframe = GtkFrame(" Read Audio Files ")
        readaudioframe.set_border_width(5)
        readaudioframe.set_shadow_type(SHADOW_ETCHED_OUT)
        readaudioframe.show()

        self.readaudiowin = GtkScrolledWindow()
        self.readaudiowin.set_policy(POLICY_AUTOMATIC, POLICY_ALWAYS)
        self.readaudiowin.show()
        
        btnReadContent = GtkButton()
        boxReadContent = self.XpmLabelBox(" Read Content ")
        boxReadContent.set_usize(95, -1)
        btnReadContent.add(boxReadContent)
        btnReadContent.connect('clicked', self.ReadAudioContent)
        btnReadContent.show()

        btnReadCD = GtkButton()
        boxReadCD = self.XpmLabelBox(" Read Songs ")
        boxReadCD.set_usize(95, -1)
        btnReadCD.add(boxReadCD)
        btnReadCD.connect('clicked', self.ReadAudioCD)
        btnReadCD.show()

        boxButton = GtkVBox(spacing = 5)
        boxButton.set_border_width(5)
        boxButton.pack_start(btnReadContent, expand = FALSE)
        boxButton.pack_start(btnReadCD, expand = FALSE)
        boxButton.show()

        tracksLabel = GtkLabel("Number of tracks:")
        tracksLabel.show()
        self.tracknumentry = GtkEntry()
        self.tracknumentry.set_usize(30, -1)
        self.tracknumentry.set_text("0")
        self.tracknumentry.set_editable(FALSE)
        self.tracknumentry.show()
        
        lengthLabel = GtkLabel("Total time:")
        lengthLabel.show()
        self.totaltime = GtkEntry()
        self.totaltime.set_usize(60, -1)
        self.totaltime.set_text("00:00:00")
        self.totaltime.set_editable(FALSE)
        self.totaltime.show()

        boxTrackInfo = GtkHBox(spacing = 5)
        boxTrackInfo.set_border_width(5)
        boxTrackInfo.pack_start(tracksLabel, expand = FALSE)
        boxTrackInfo.pack_start(self.tracknumentry, expand = FALSE)
        boxTrackInfo.pack_start(lengthLabel, expand = FALSE)
        boxTrackInfo.pack_start(self.totaltime, expand = FALSE)
        boxTrackInfo.show()
        
        boxCDInfo = GtkHBox(spacing = 5)
        boxCDInfo.set_border_width(5)
        boxCDInfo.pack_start(self.readaudiowin)
        boxCDInfo.pack_start(boxButton, expand = FALSE)
        boxCDInfo.show()

        self.boxTracks = GtkVBox(spacing = 2)
        self.boxTracks.set_border_width(5)
        self.boxTracks.show()

        self.readaudiowin.add_with_viewport(self.boxTracks)
    
        boxReadAudioCD = GtkVBox()
        boxReadAudioCD.pack_start(boxTrackInfo, expand = FALSE)
        boxReadAudioCD.pack_start(boxCDInfo)
        boxReadAudioCD.show()
        
        readaudioframe.add(boxReadAudioCD)
        
        readCDbox.pack_start(readaudioframe)

        self.audiotracklist = []
        
	frame.add(readCDbox)

    def ReadAudioCD(self, args):
        self.writermode = modeReadAudioCD
        command = "cdda2wav -D %d,%d,%d -x -B -H -g -O wav -t %%d+%%d \"%%s/Track-%%d.wav\" 2>&1 | %s" % ( int(self.channelReader.get_text()), int(self.idReader.get_text()), int(self.lunReader.get_text()), ecat)
        self.BurnWindow(command)

    # Get Content of a Audio-CD
    def ReadAudioContent(self, args):
        command = "cdda2wav -D %d,%d,%d -N -J -v 2 2>&1" % ( int(self.channelReader.get_text()), int(self.idReader.get_text()), int(self.lunReader.get_text()))
        pipe = popen2.popen2(command)
        line = None
        boolTrackInfo = False
        self.audiotracklist = []
        self.readaudiowin.remove(self.readaudiowin.children()[0])
        self.boxTracks = GtkVBox(spacing = 2)
        self.boxTracks.set_border_width(5)
        while (line != ""):
            line = pipe[0].readline()
            if (line != ""):
                stripped = strip(line)
                if lower(stripped[0:18]) == "table of contents:":
                    posTrackNum = find(stripped, "total tracks:") + 13
                    posTrackNumEnd = find(stripped[posTrackNum:], ",") + posTrackNum
                    numberOfTracks = stripped[posTrackNum:posTrackNumEnd]
                    self.tracknumentry.set_text(numberOfTracks)
                    posTotalTime = find(stripped, "total time ") + 11
                    posTotalTimeEnd = find(stripped[posTotalTime:], ")") + posTotalTime
                    totalTime = stripped[posTotalTime:posTotalTimeEnd]
                    self.totaltime.set_text(totalTime)
                    boolTrackInfo = True
                elif (boolTrackInfo == True):
                    if stripped[0] in digits:
                        songs = split(stripped, ",")
                        for intCounter in range(len(songs)):
                            song = strip(songs[intCounter])
                            if (song != ""):
                                tracknum = atoi(strip(song[:find(song, ".")]))
                                length = strip(song[find(song, "(") + 1:find(song, ")")])
                                trackString = "Track: %s - Length: %s" % (zfill(tracknum, 2), length)
                                self.audiotracklist.append(GtkCheckButton(trackString))
                                self.audiotracklist[tracknum - 1].show()
                                self.boxTracks.pack_start(self.audiotracklist[tracknum - 1])
                    else:
                        boolTrackInfo = False
        self.boxTracks.show()
        self.readaudiowin.add_with_viewport(self.boxTracks)
        
        pipe[0].close()
        pipe[1].close()
        pipe = None
        
    # Create a frame for copying a CD
    def CopyCDFrame(self, frame):

        labwarning = GtkLabel("Direclty copying a CD might not be a good idea, if your computer " +
                              "is slow, the source CD is scratched, \nthe magnetic radiation of your heater is " +
                              "too extreme or your wife has a bad day, so everything you do here is at \nyour " +
                              "own risk !")
        labwarning.set_justify(JUSTIFY_LEFT)
        labwarning.show()

        frmWarning = GtkFrame(" Warning ")
	frmWarning.set_shadow_type(SHADOW_ETCHED_OUT)
        frmWarning.add(labwarning)
        frmWarning.show()
        
        labLeft = GtkLabel()
        labLeft.show()
        labRight = GtkLabel()
        labRight.show()

        btnPerformCopy = GtkButton()
	box = self.XpmLabelBox(" Perform direct CD-copy ", "%s/burn.xpm" % iconpath)
	btnPerformCopy.add(box)
        btnPerformCopy.connect('clicked', self.BurnCD)
        btnPerformCopy.show()
        
        tabCopyCD = GtkTable(1, 3)
        tabCopyCD.attach(labLeft, 0, 1, 0, 1, EXPAND, 0, 5, 5)
        tabCopyCD.attach(btnPerformCopy, 1, 2, 0, 1, 0, 0, 5, 5)
        tabCopyCD.attach(labRight, 2, 3, 0, 1, EXPAND, 0, 5, 5)
        tabCopyCD.show()

        frmBurn = GtkFrame()
        frmBurn.set_shadow_type(SHADOW_ETCHED_OUT)
        frmBurn.add(tabCopyCD)
        frmBurn.show()
        
        copyCDbox = GtkVBox(spacing = 5)
        copyCDbox.set_border_width(5)
        copyCDbox.pack_start(frmWarning, expand = FALSE)
        copyCDbox.pack_start(frmBurn, expand = FALSE)
        copyCDbox.show()

	frame.add(copyCDbox)

    # Read a CD
    def ReadImage(self, args):
        readimage = self.readcdimage.gtk_entry().get_text()
        if (readimage == ""):
            GnomeErrorDialog("Please enter a filename for the image file first.")
            return

        command = "readcd -dev=%d,%d,%d f=\"%s\" 2>&1 | %s" % ( int(self.channelReader.get_text()), int(self.idReader.get_text()), int(self.lunReader.get_text()), readimage, ecat)
        self.writermode = modeReadCD
        self.BurnWindow(command)

    # Returns the length of a mp3/wav file in seconds
    def GetAudioTrackLength(self, filename):
        if (lower(filename[-4:]) == ".mp3"):
            mpeginfo = mp3info.MP3Info(filename)
            mpeginfo.ReadData()
            seconds = int(mpeginfo.MPEGData["Duration"])
            del mpeginfo
        else:
            wavefile = wave.open(filename, "r")
            seconds = wavefile.getnframes() / wavefile.getframerate ()
            wavefile.close()
        return seconds
    
    # Select all files either in the audio track or data track window
    def SelectAll(self, args):
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.datafiles.select_all()
        elif (frames[page] == audiocd):
            self.songlist.select_all()
        elif (frames[page] == readcd):
            for i in range(len(self.audiotracklist)):
                self.audiotracklist[i].set_active(1)

    # UnSelect all files either in the audio track or data track window
    def UnSelectAll(self, args):
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.datafiles.unselect_all()
        elif (frames[page] == audiocd):
            self.songlist.unselect_all()
        elif (frames[page] == readcd):
            for i in range(len(self.audiotracklist)):
                self.audiotracklist[i].set_active(0)
        
    # Main loop
    def main(self):
        rc_parse(gtkrcfile)
	mainloop()

