"""

Miscellaneous functions required by eroaster

"""

from string import strip, lower, split, zfill
from os import getcwd, chdir, mkdir, wait, listdir, lstat
from types import IntType, StringType

import signal
import os

# Beautify 0/1 -> True / False and vice versa
def beautify(value):
    if type(value) == IntType:
        if value == 0:
            return "False"
        elif value == 1:
            return "True"
    elif type(value) == StringType:
        if lower(value) == "true":
            return 1
        elif lower(value) == "false":
            return 0

# Handler, which waits for children to be closed
def SigChldHandler(signal_number, stack_frame):
    # Reinstall the sigchild handler, cuz it gets deleted on libc5/sysv machines
    signal.signal(signal.SIGCHLD, SigChldHandler)
    try:
        result = wait()
    except:
        pass

# Check wether a directory exists and create it if not
def CheckDirectory(directory):
    if (directory[-1:] == "/"):
        directory = directory[:-1]
    if (not os.path.exists(directory)):
        pwd = getcwd()
        splitted = split(directory, "/")
        path = "/"
        for i in range(len(splitted)):
            path = "%s/%s" % (path, splitted[i])
            if not os.path.exists(path):
                mkdir(path)
        chdir(pwd)

# Strip each value of a list
def striplist(list):
    for i in range(len(list)):
        list[i] = strip(list[i])
    return list

# Converts time in seconds to a nicely formatted string
def nicetime(seconds):
    minutes, seconds = divmod(seconds, 60)
    return "%s:%s" % (zfill(minutes, 2), zfill(seconds, 2))

# Calculate the disk usage of a given directory
def du(file):
    if (os.path.islink(file)) or (os.path.isfile(file)):
        size = lstat(file)[6]
    else:
        filelist = listdir(file)
        size = 0
        for i in range(len(filelist)):
            size = size + du(file + "/" + filelist[i])
    return size
